import React, { Component } from "react";
import { Route, Switch } from "react-router-dom";
import { BrowserRouter as Router, Redirect } from "react-router-dom";
import { ToastContainer, toast } from "react-toastify";
import LoadingOverlay from "react-loading-overlay";
import PrivateRoute from "./components/routes/PrivateRoute";
import LoginRoute from "./components/routes/LoginRoute";
import { client, customEvents } from "./functions/eventEmitter";
import HomePage from "./components/pages/Home";
import User from "./components/pages/User";
import LogIn from "./components/pages/LogIn";
import SignUp from "./components/pages/SignUp";
import { Provider } from "react-redux";

import { ConnectedRouter } from "connected-react-router";

import configureStore, { history } from "./configureStore";
import storage from "./modules/storage";
import userAction from "./actions/user";
import appAction from "./actions/app";
import api from "./modules/api";
import SpeechReport from "./components/pages/SpeechReport";
// import 'antd/dist/antd.css';
// import 'antd/es/icon/style';

export const store = configureStore();
store.subscribe(() => {
  console.log("change state", store.getState());
});

class App extends Component {
  constructor(props) {
    super(props);
    const user = storage.loadUser();
    console.log("PREVIOUS USER", user);
    const isLogin = user && user._id;
    if (user && user._id) {
      // store.dispatch(userAction.updateUser(user))
      store.dispatch(appAction.changeField("isLogin", true));
      store.dispatch(userAction.updateUser(user));
      api.setToken(user.access_token);
    }
    // this.userAuth = new UserAuth();
    this.state = {
      isLogin,
    };
    store.subscribe(() => {
      this.handleNewState(store.getState());
    });
  }

  handleNewState(state) {
    if (this.state.isLogin !== state.app.isLogin) {
      this.setState({ isLogin: state.app.isLogin });
    }
  }

  componentDidMount() {
    client.on(customEvents.loginSuccess, () => {
      // this.showSuccess("Đăng nhập thành công!");
    });
    client.on(customEvents.loginFail, () => {
      this.showError(
        "Đăng nhập thất bại!\nTài khoản hoặc mật khẩu không đúng."
      );
    });
    client.on(customEvents.loginError, () => {
      this.showError("Có lỗi xảy ra. Vui lòng thử lại.");
    });

    client.on(customEvents.signupSuccess, () => {
      this.showSuccess("Đăng ký tài khoản mới thành công! Vui lòng đăng nhập.");
    });
    client.on(customEvents.signupFail, () => {
      this.showError(
        "Tài khoản đã tồn tại hoặc có lỗi xảy ra!\nVui lòng thử lại."
      );
    });
    client.on(customEvents.signupError, () => {
      this.showError("Có lỗi xảy ra.\nVui lòng thử lại.");
    });

    client.on(customEvents.reportSuccess, () => {
      this.showSuccess("Gửi báo cáo thành công!");
    });
    client.on(customEvents.reportFail, () => {
      this.showError("Gửi báo cáo thất bại!\nVui lòng thử lại.");
    });
    client.on(customEvents.reportError, () => {
      this.showError("Có lỗi xảy ra.\nVui lòng thử lại.");
    });

    client.on(customEvents.timePredictSuccess, () => {
      this.showSuccess("Hiển thị tình trạng giao thông dự đoán...");
    });
    client.on(customEvents.timePredictError, () => {
      this.showError("Thời gian thiếu hoặc không hợp lệ!");
    });

    client.on(customEvents.timeReportSuccess, () => {
      this.showSuccess(
        "Hiển thị tình trạng giao thông báo cáo từ người dùng thành công"
      );
    });

    client.on(customEvents.redirectSuccess, () => {
      this.showSuccess("Tìm đường thành công");
    });
    client.on(customEvents.redirectError, () => {
      this.showError("Tìm đường thất bại");
    });
  }

  componentWillUnmount() {
    client.removeEventListener(customEvents.getSuccess, this.showSuccess);
  }

  logout() {
    this.userAuth.logout();
    this.setState({
      user: false,
    });
  }

  showSuccess(message) {
    toast.success(message);
  }

  showFail(message) {
    toast.warn(message);
  }

  showError(message) {
    toast.error(message);
  }

  render() {
    const { isLogin } = this.state;
    console.log("ISLOGIN", isLogin);
    if (isLogin) {
      return (
        <Provider store={store}>
          <ConnectedRouter history={history}>
            <Router basename={"/utraffic"}>
              <LoadingOverlay>
                <Switch>
                  <Route exact path="/" component={HomePage} />
                  <PrivateRoute exact path="/user" component={User} />
                  <PrivateRoute exact path="/speech-report" component={SpeechReport} />
                  <Redirect to="/" />
                </Switch>
                <ToastContainer />
              </LoadingOverlay>
            </Router>
          </ConnectedRouter>
        </Provider>
      );
    } else {
      return (
        <Provider store={store}>
          <ConnectedRouter history={history}>
            <Router basename={"/utraffic"}>
              <LoadingOverlay>
                <Switch>
                  <Route exact path="/" component={HomePage} />
                  <Route path="/login" component={LogIn} />
                  {/* <LoginRoute
                    exact
                    path="/login"
                    component={LogIn}
                    isLogin={isLogin}
                  /> */}
                  <LoginRoute exact path="/signup" component={SignUp} />
                  <PrivateRoute exact path="/user" component={User} />
                  <PrivateRoute exact path="/speech-report" component={SpeechReport} />
                  <Redirect to="/" />
                </Switch>
                <ToastContainer />
              </LoadingOverlay>
            </Router>
          </ConnectedRouter>
        </Provider>
      );
    }
  }
}

// export default () => {
//   return <div></div>;
// };

export default App;
