import { createAction } from "./app";
import api from "../modules/api";
import storage from "../modules/storage";
import AppAction from "./app";
import { userActions } from "./user";

export const Types = {
  changeField: "auth.change-field",
  changeFormField: "auth.change-form-field",
  cleanForm: "auth.form.clean",

  registerRequest: "auth.register.request",
  registerSuccess: "auth.register.success",
  registerFailure: "auth.register.failure",

  loginRequest: "auth.login.request",
  loginSuccess: "auth.login.success",
  loginFailure: "auth.login.failure",

  logout: "auth.logout",
};

const isLogin = () => (dispatch, getState) => {
  const state = getState();
  return state.app.isLogin;
};

const cleanForm = () => createAction(Types.cleanForm);

const changeField = (key = "", value) =>
  createAction(Types.changeField, { key, value });

const changeFormField = (key = "", value) =>
  createAction(Types.changeFormField, { key, value });

const register = (data) => (dispatch, getState) => {
  dispatch(createAction(Types.registerRequest));
  return api.register(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.registerSuccess, response.data));
      return true;
    } else {
      dispatch(
        createAction(Types.registerFailure, {}, "Đăng ký tài khoản thất bại")
      );
      return false;
    }
  });
};

// const loginWithFaceBook = (data) => dispatch =>{

// }
/**
 *
 * @param {*} data
 * @param {'basic' | 'google'} type
 */
const login = (data, type) => (dispatch, getState) => {
  dispatch(createAction(Types.loginRequest));
  return api.login(data, type).then((response) => {
    if (response.success) {
      storage.saveUser(response.data);
      api.setToken(response.data.access_token);
      dispatch(AppAction.changeField("isLogin", true));
      dispatch(userActions.updateUser(response.data));
      dispatch(createAction(Types.loginSuccess));
      // dispatch(cleanForm());
      return true;
    } else {
      dispatch(createAction(Types.loginFailure, null, "Đăng nhập thất bại"));
    }
    return false;
  });
};

const logout = () => (dispatch) => {
  storage.removeUser();
  api.setToken(null);
  dispatch(AppAction.changeField("isLogin", false));
  dispatch(userActions.updateUser({}));
  dispatch(AppAction.resetAll());
};

export default {
  Types,
  changeField,
  changeFormField,
  register,
  cleanForm,
  login,
  logout,
  isLogin,
};
