import { createAction } from './app';
// import _ from 'lodash';
import { v4 as uuidv4 } from 'uuid';
import api from '../modules/api';
import constants from '../constants';
import {
  onRedirect,
  onRedirectReport,
} from '../modules/mapFunctions';
import errorHandle from '../components/shared/errorHandle';

export const Types = {
  RedirectRequest: 'redirect-request',
  RedirectFail: 'redirect-fail',
  RedirectSuccess: 'redirect-success',

  DirectReportRequest: 'direct-report-request',
  DirectReportFail: 'direct-report-fail',
  DirectReportSuccess: 'direct-report-success',

  setState: 'direct.set-state',
};

const redirect = (map, query) => (dispatch, getState) => {
  const priority = getState().redirect.priority;
  dispatch(createAction(Types.RedirectRequest));
  return api.redirect(query).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.RedirectSuccess, response.data));
      onRedirect(map, response.data, query, priority);
      return true;
    } else {
      dispatch(createAction(Types.RedirectFail, {}, 'tìm đường thất bại'));
      return false;
    }
  });
};

/**
 *
 * @param {boolean} drawable - Check whether map is use in draw mode or pick mode
 * @param {function} clearMapFunc - Function for clear the map
 * @param {function} setReportState - Set report state
 * @param {array} multiReportData - Data from multi report
 */

const handleDirectReportError = (
  drawable,
  clearMapFunc,
  setReportState,
) => {
  if (!drawable) {
    setReportState({
      'report.markerData': [],
      'report.addressData': [
        constants.addressDataStartDefault,
        constants.addressDataEndDefault,
      ],
    });
    clearMapFunc();
  } 

};

/**
 *
 * @param {*} map - Map for drawing direct
 * @param {object} query - Query for direct api
 * @param {boolean} drawable - Check whether map is use in draw mode or pick mode
 * @param {function} clearMapFunc -Function for clear the map
 * @param {function} setReportState - Set report state
 * @returns {boolean}
 */

const directReport =
  (map, query, drawable, clearMapFunc, setReportState) =>
  (dispatch, getState) => {
    const priority = getState().redirect.priorityReport;
    const multiReportData = getState().trafficStatus.report.multiReportData;
    dispatch(createAction(Types.DirectReportRequest));
    return api.redirect(query).then((response) => {
      if (response.success && response.data.length > 0) {
        dispatch(createAction(Types.DirectReportSuccess, response.data));
        if (response.data[0].distance > 1000) {
          handleDirectReportError(
            drawable,
            clearMapFunc,
            setReportState,
          );
          errorHandle('Khoảng cách gửi báo cáo không được quá 1km');
        } else {
          // delete reportError.distance;
          if (drawable && multiReportData.length >= 5) {
            errorHandle('Không được chọn hơn 5 đoạn đường');
            return true;
          }

          const segmentIdList = response.data[0].coords.map(item => item.segment_id);
          setReportState({
            'report.multiReportData': [
              ...multiReportData,
              {
                segments: segmentIdList,
                id : uuidv4()
              }
            ]
          });
          onRedirectReport(map, response.data, query, priority, drawable);
        }
        return true;
      } else {
        dispatch(
          createAction(Types.DirectReportFail, {}, 'tìm đường thất bại')
        );
        handleDirectReportError(
          drawable,
          clearMapFunc,
          setReportState,
        );
        errorHandle('Không thể tìm đường giữa hai điểm này');
        return false;
      }
    });
  };

const setState = (state) => createAction(Types.setState, state);

export default {
  Types,
  redirect,
  setState,
  directReport,
};
