import { createAction } from './app';

import api from '../modules/api';
import { onGetTrafficStatus } from '../modules/mapFunctions';
import { utils } from '../modules/utils';
import errorHandle from '../components/shared/errorHandle';

export const Types = {
  postReportRequest: 'post-report-request',
  postReportFail: 'post-report-fail',
  postReportSuccess: 'post-report-success',

  getTrafficStatusRequest: 'get-traffic-status-request',
  getTrafficStatusFail: 'get-traffic-status-fail',
  getTrafficStatusSuccess: 'get-traffic-status-success',

  cleanState: 'clean-traffic-status-state',
  changeField: 'change-filed-traffic-status',

  cancelGetTrafficStatus: 'traffic-status.cancel-request',
};

const postSpeechReport = (data) => (dispatch) => {
  dispatch(createAction(Types.postReportRequest));
  return api.postSpeechReport(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.postReportSuccess, response.data));
      return true;
    } else {
      const errors = response.errors;
      const message = utils.getMessageErrors(errors.message, errors.errors);
      errorHandle('Gửi báo cáo thất bại', errors.message);
      dispatch(createAction(Types.postReportFail, {}, message));
      return false;
    }
  });
};

const postReport = (data) => (dispatch) => {
  dispatch(createAction(Types.postReportRequest));
  return api.postReport(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.postReportSuccess, response.data));
      return true;
    } else {
      const errors = response.errors;
      const message = utils.getMessageErrors(errors.message, errors.errors);
      dispatch(createAction(Types.postReportFail, {}, message));
      return false;
    }
  });
};

const postReportV2 = (data) => (dispatch) => {
  dispatch(createAction(Types.postReportRequest));
  return api.postReportV2(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.postReportSuccess, response.data));
      return true;
    } else {
      const errors = response.errors;
      const message = utils.getMessageErrors(errors.message, errors.errors);
      // const message = {message: 'Gửi báo cáo thất bại', notice: response.errors.message}
      errorHandle('Gửi báo cáo thất bại', response.errors.message);
      dispatch(createAction(Types.postReportFail, {}, message));
      return false;
    }
  });
};

const postMultiReport = (data) => (dispatch) => {
  dispatch(createAction(Types.postReportRequest));
  return api.postMultiReport(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.postReportSuccess, response.data));
      return true;
    } else {
      const errors = response.errors;
      const message = utils.getMessageErrors(errors.message, errors.errors);
      // const message = {message: 'Gửi báo cáo thất bại', notice: response.errors.message}
      errorHandle('Gửi báo cáo thất bại', response.errors.message);
      dispatch(createAction(Types.postReportFail, {}, message));
      return false;
    }
  }).catch((err) => {
    console.error(err);
  })
};

const postFastReport = (data) => (dispatch) => {
  dispatch(createAction(Types.postReportRequest));
  return api.postFastReport(data).then((response) => {
    if (response.success) {
      dispatch(createAction(Types.postReportSuccess, response.data));
      return true;
    } else {
      const errors = response.errors;
      const message = utils.getMessageErrors(errors.message, errors.errors);
      // const message = {message: 'Gửi báo cáo thất bại', notice: response.errors.message}
      errorHandle('Gửi báo cáo thất bại', response.errors.message);
      dispatch(createAction(Types.postReportFail, {}, message));
      return false;
    }
  }).catch((err) => {
    console.error(err);
  })
};

let previousGetTrafficRequest;
const getTrafficStatus =
  (map, zoom, darkMode, query, isV2 = false, getAvailable = false) =>
    (dispatch, getState) => {
      if (previousGetTrafficRequest) {
        try {
          previousGetTrafficRequest.abort();
        } catch (error) {
          console.log('ERROR WHEN CANCEL REQUEST', error);
        }
      }
      const state = getState();
      const { trafficStatus, isDrew } = state.trafficStatus;
      const { data } = trafficStatus;

      if (zoom <= 13) {
        dispatch(changeField({ isDrew: false, isFetching: false }));
        return Promise.resolve(true);
      } else if (!isDrew) {
        onGetTrafficStatus(map, data, zoom, darkMode);
        dispatch(changeField({ isDrew: true }));
      }
      // onGetTrafficStatus(map, data, zoom, darkMode);
      // }
      dispatch(createAction(Types.getTrafficStatusRequest));
      previousGetTrafficRequest = api.getTrafficStatus(query, true, isV2);
      return previousGetTrafficRequest.fetchData().then((response) => {
        if (response.success) {
          dispatch(createAction(Types.getTrafficStatusSuccess, response.data));
          onGetTrafficStatus(map, response.data, zoom, darkMode);
          return true;
        } else {
          console.log('RESPONSE ERROR', response);
          dispatch(
            createAction(
              Types.getTrafficStatusFail,
              { code: response.errors && response.errors.code },
              'Lấy tình trạng giao thông thất bại'
            )
          );
          return false;
        }
      });
    };

const cancelGetTrafficStatus = () => (dispatch) => {
  dispatch(createAction(Types.cancelGetTrafficStatus));
  if (previousGetTrafficRequest) {
    previousGetTrafficRequest.abort();
  }
};

const cleanState = (stateName) => createAction(Types.cleanState, { stateName });
const changeField = (state) => createAction(Types.changeField, { state });

export default {
  Types,
  postSpeechReport,
  postReport,
  postReportV2,
  postMultiReport,
  postFastReport,
  getTrafficStatus,
  cleanState,
  changeField,
  cancelGetTrafficStatus,
};
