import React from "react";
import { Formik, Form, ErrorMessage } from "formik";
import {
  MDBCol,
  MDBBtn,
  MDBInput,
  MDBIcon,
} from "mdbreact";
import { Route, Redirect, Link } from "react-router-dom";
import FacebookLogin from "react-facebook-login/dist/facebook-login-render-props";
import GoogleLogin from "react-google-login";
import { client, customEvents } from "../../functions/eventEmitter";
// import userAction from '../../actions/user';
import authAction from "../../actions/auth";
import { connect } from "react-redux";
import { Spin } from "antd";
// import Api from '../../modules/api';

class LogInForm extends React.Component {
  constructor(props) {
    super(props);
    let user = localStorage.getItem("user");
    this.state = {
      login: user ? true : false,
    };
  }

  // async login(response, type) {
  //   let result = await this.api.loginViaWebService(response, type);
  //   if (result) {
  //     client.emit(customEvents.loginSuccess);
  //     this.setState({
  //       login: true
  //     })
  //   } else {
  //     client.emit(customEvents.loginFail);
  //   }
  // }

  renderRedirect = () => {
    if (this.state.login) {
      return <Redirect to="/user" />;
    }
  };

  onLogin = (res, type) => {
    let data = {};
    if (type === "google") {
      const { tokenId, profileObj } = res;
      const { googleId } = profileObj;
      data = {
        google_id: googleId,
        google_token: tokenId,
      };
    }
    if (type === "facebook") {
      const { accessToken, id } = res;
      data = {
        facebook_id: id,
        facebook_token: accessToken,
      };
    }
    this.props.login(data, type);
  };

  render() {
    const { login } = this.state;
    const { auth } = this.props;
    if (login) return <Redirect to="/user" />;
    return (
      <Route
        render={({ history }) => (
          <Formik
            initialValues={{ username: "", password: "" }}
            validate={(values) => {
              let errors = {};
              if (!values.username) {
                errors.username = "*Nhập tên tài khoản";
              } else if (values.username.length < 5) {
                errors.username = "*Tên tài khoản cần ít nhất 5 kí tự";
              }
              if (!values.password) {
                errors.password = "*Nhập mật khẩu";
              }
              return errors;
            }}
            onSubmit={async (values, { setSubmitting }) => {
              this.props.login(values).then((success) => {
                if (success) {
                  client.emit(customEvents.loginSuccess);
                } else {
                  client.emit(customEvents.loginFail);
                }
              });
            }}
          >
            {({ values, touched, handleBlur, handleChange, isSubmitting }) => (
              <Form className="w-100">
                <div className="row justify-content-between">
                  <h4 className="teal-text mb-4 default-font">Đăng nhập</h4>
                  <span
                    style={{
                      marginTop: "-24px",
                    }}
                  >
                    <Link
                      to="/"
                      style={{
                        padding: "8px",
                        cursor: "pointer",
                      }}
                    >
                      <i class="fa fa-times" aria-hidden="true"></i>
                    </Link>
                  </span>
                </div>
                <div className="grey-text col-md-10 offset-md-1 col-sm-12">
                  <MDBInput
                    name="username"
                    label="Tài khoản"
                    icon="user"
                    group
                    type="text"
                    onChange={handleChange}
                    onBlur={handleBlur}
                    value={values.username}
                  ></MDBInput>
                  <ErrorMessage
                    className="small red-text"
                    name="username"
                    component="div"
                  ></ErrorMessage>
                  <MDBInput
                    name="password"
                    label="Mật khẩu"
                    icon="lock"
                    group
                    type="password"
                    onChange={handleChange}
                    onBlur={handleBlur}
                    value={values.password}
                  ></MDBInput>
                  <ErrorMessage
                    className="small red-text"
                    name="password"
                    component="div"
                  ></ErrorMessage>
                </div>

                <p className="text-center small grey-text mt-3 mb-3">
                  Chưa có tài khoản?
                  <span className="mx-1">
                    <Link to="/signup">Đăng ký</Link>
                  </span>
                  mới ngay!
                </p>

                <div className="text-center">
                  <MDBBtn
                    type="submit"
                    disabled={auth.isFetching}
                    style={{
                      borderRadius: "24px",
                    }}
                  >
                    <Spin className="mx-1" spinning={auth.isFetching}></Spin>
                    Đăng nhập
                  </MDBBtn>
                </div>

                <MDBCol size="12" className="mb-1">
                  <hr></hr>
                  <p>Hoặc đăng nhập với:</p>
                </MDBCol>
                <MDBCol size="12" xs="6" className="text-center">
                  <GoogleLogin
                    // clientId="990709992068-2i99g5echg7d1mrotb6aguqbg2dj7rto.apps.googleusercontent.com"
                    // clientId="131064076267-l5mfjlktr38bqu935n32ima5u7vu5b2d.apps.googleusercontent.com"
                    clientId="172026625059-pbi38sk8r992dt70024uef9p6rqmefcs.apps.googleusercontent.com"
                    onSuccess={(res) => {
                      this.onLogin(res, "google");
                    }}
                    onFailure={(res) => {
                      console.log("GOOGLE FAIL", res);
                    }}
                    cookiePolicy={"single_host_origin"}
                    render={(renderProps) => (
                      <button
                        className="btn btn-login-with btn-google"
                        type="button"
                        style={{
                          borderRadius: "24px",
                        }}
                        onClick={renderProps.onClick}
                        disabled={renderProps.disabled}
                      >
                        <div className="center">
                          <MDBIcon icon="google" className="btn-icon"></MDBIcon>
                          <span>Google</span>
                        </div>
                      </button>
                    )}
                  />
                </MDBCol>
                <MDBCol size="12" xs="6" className="text-center">
                  <FacebookLogin
                    // appId="637465913375868"
                    appId="1592356364232559"
                    fields="name,email"
                    callback={(res) => {
                      console.log("FACEBOOK CALLBACK", res);
                      this.onLogin(res, "facebook");
                    }}
                    render={(renderProps) => (
                      <button
                        className="btn btn-login-with btn-facebook"
                        type="button"
                        style={{
                          borderRadius: "24px",
                        }}
                        onClick={renderProps.onClick}
                      >
                        <div className="center">
                          <MDBIcon
                            icon="facebook"
                            className="btn-icon"
                          ></MDBIcon>
                          <span>Facebook</span>
                        </div>
                      </button>
                    )}
                  />
                </MDBCol>
                <MDBCol xs="12" className="mt-4">
                  <Link to="/" className="grey-text small">
                    &larr; Trở về Trang chủ
                  </Link>
                </MDBCol>
              </Form>
            )}
          </Formik>
        )}
      />
    );
  }
}

const mapStateToProps = (state) => ({
  auth: state.auth,
});

const mapDispatchToProps = {
  login: authAction.login,
};

export default connect(mapStateToProps, mapDispatchToProps)(LogInForm);
