import React from "react";
import { Formik, Form, ErrorMessage } from "formik";
import { MDBBtn, MDBInput } from "mdbreact";
import { Container, Row, Col } from "reactstrap";
import { Route, Link, Redirect } from "react-router-dom";
// import SignUpAPI from '../../api/signup';
// import { isExisted } from '../../functions/common';
import { client, customEvents } from "../../functions/eventEmitter";
import { connect } from "react-redux";
import authAction from "../../actions/auth";
import api from "../../modules/api";
import _ from "lodash";
import { Spin } from "antd";

class SignUpForm extends React.Component {
  constructor() {
    super();
    // this.api   = new SignUpAPI();
    this.state = {
      signup: false,
    };
  }

  renderRedirect = () => {
    if (this.state.signup) {
      return <Redirect to="/login" />;
    }
  };

  render() {
    const { auth } = this.props;
    return (
      <Route
        render={({ history }) => (
          <Formik
            initialValues={{
              username: "",
              password: "",
              confirmpassword: "",
              phone: "",
            }}
            validate={(values) => {
              const phoneRegex = /^0[3|5|7|8|9][0-9]{8}$/;
              let errors = {};
              if (!phoneRegex.test(values.phone) && values.phone) {
                errors.phone = "*Số điện thoại không đúng định dạng";
              }
              if (!values.username) {
                errors.username = "*Nhập tên tài khoản";
              } else if (values.username.length < 5) {
                errors.username = "*Tên tài khoản cần ít nhất 5 kí tự";
              }
              if (!values.password) {
                errors.password = "*Nhập mật khẩu";
              } else if (values.password.length < 5) {
                errors.password = "Mật khẩu cần ít nhất 5 kí tự";
              }
              if (!values.confirmpassword) {
                errors.confirmpassword = "*Xác nhận lại mật khẩu";
              } else if (values.confirmpassword !== values.password) {
                errors.confirmpassword = "*Mật khẩu xác nhận không khớp";
              }
              return errors;
            }}
            onSubmit={(values, { setSubmitting }) => {
              let file = values.file;
              if (!file) {
                this.props.register(values).then((success) => {
                  if (success) {
                    this.setState({ signup: true });
                    client.emit(customEvents.signupSuccess);
                  } else {
                    client.emit(customEvents.signupFail);
                  }
                });
              } else {
                api.uploadFile(file).then((response) => {
                  if (response.success) {
                    values = _.omit(values, ["file"]);
                    values.avatar = response.data;
                    this.props.register(values).then((success) => {
                      if (success) {
                        this.setState({ signup: true });
                        client.emit(customEvents.signupSuccess);
                      } else {
                        client.emit(customEvents.signupFail);
                      }
                    });
                  } else {
                    console.log("UPLOAD IMAGE FAIL", response);
                  }
                });
              }
            }}
          >
            {({
              values,
              touched,
              handleBlur,
              handleChange,
              isSubmitting,
              setFieldValue,
            }) => (
              <Form>
                {this.renderRedirect()}
                <Container>
                  <Row>
                    <Col xs="12">
                      <div className="row justify-content-between">
                        <h4 className="teal-text mb-4 default-font">Đăng ký</h4>
                        <span
                          style={{
                            marginTop: "-24px",
                            marginRight: '-16px',
                          }}
                        >
                          <Link
                            to="/"
                            style={{
                              padding: "8px",
                              cursor: "pointer",
                            }}
                          >
                            <i class="fa fa-times" aria-hidden="true"></i>
                          </Link>
                        </span>
                      </div>
                    </Col>
                    <Col lg="1" xs="12"></Col>
                    <Col lg="5" xs="12" className="grey-text mb-5">
                      <p className="small">Thông Tin Đăng Nhập (bắt buộc)</p>
                      <MDBInput
                        name="username"
                        label="Tài khoản *"
                        icon="user"
                        type="text"
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.username}
                      />
                      <ErrorMessage
                        className="small text-right red-text"
                        name="username"
                        component="div"
                      />
                      <MDBInput
                        name="password"
                        label="Mật khẩu *"
                        icon="lock"
                        type="password"
                        group
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.password}
                      />
                      <ErrorMessage
                        className="small text-right red-text"
                        name="password"
                        component="div"
                      />
                      <MDBInput
                        name="confirmpassword"
                        label="Xác nhận mật khẩu *"
                        icon="lock"
                        type="password"
                        group
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.confirmpassword}
                      />
                      <ErrorMessage
                        className="small text-right red-text"
                        name="confirmpassword"
                        component="div"
                      />
                    </Col>
                    <Col lg="1" xs="12"></Col>
                    <Col lg="4" xs="12" className="account-info grey-text mb-4">
                      <p className="small">Thông Tin Tài Khoản</p>
                      <MDBInput
                        name="name"
                        label="Họ tên"
                        icon="user"
                        type="text"
                        group
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.name}
                      />
                      <MDBInput
                        name="email"
                        label="Email"
                        icon="envelope"
                        type="email"
                        group
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.email}
                      />
                      <MDBInput
                        name="phone"
                        label="Số điện thoại"
                        icon="phone"
                        type="text"
                        group
                        onChange={handleChange}
                        onBlur={handleBlur}
                        value={values.phone}
                      />
                      <ErrorMessage
                        className="small text-right red-text"
                        name="phone"
                        component="div"
                      />
                      <MDBInput
                        name="file"
                        icon="image"
                        type="file"
                        group
                        className="upload-file"
                        onChange={(event) => {
                          setFieldValue("file", event.currentTarget.files[0]);
                        }}
                        onBlur={handleBlur}
                        // label="file"
                        // label="Upload ảnh đại diện"
                      />
                    </Col>
                    <Col xs="12" className="text-center">
                      <p className="small grey-text mt-3 mb-3">
                        Đã có tài khoản?{" "}
                        <span>
                          <Link to="/login">Đăng nhập</Link>
                        </span>{" "}
                        ngay!
                      </p>
                      <MDBBtn
                        type="submit"
                        style={{
                          borderRadius: "24px",
                        }}
                      >
                        <Spin
                          className="mx-2"
                          spinning={auth.isFetching}
                        ></Spin>{" "}
                        Đăng ký
                      </MDBBtn>
                    </Col>
                    <Col xs="12" className="mt-4">
                      <Link to="/" className="grey-text small">
                        &larr; Trở về Trang chủ
                      </Link>
                    </Col>
                  </Row>
                </Container>
              </Form>
            )}
          </Formik>
        )}
      />
    );
  }
}

const mapSateToProps = (state) => ({
  auth: state.auth,
});

const mapDispatchToProps = {
  register: authAction.register,
};

export default connect(mapSateToProps, mapDispatchToProps)(SignUpForm);
