import React, { createRef } from 'react';
import { Map, TileLayer, Marker, Tooltip } from 'react-leaflet';
import {
  weightPolyline,
  gradient,
} from './fragments/DrawTools';
import L from 'leaflet';
import { connect } from 'react-redux';
import trafficStatusAction from '../../actions/traffic-status';
import redirectAction from '../../actions/redirect';
import locationIcon from '../../static/images/location.png';
import locationIcon2 from '../../static/images/location2.png';
import locateIconImg from '../../static/images/circle.png';
import './style.css';
import {
  Icon,
  Row,
  Col,
  Button,
  Tooltip as AntTooltip,
  Switch,
  Cascader,
  Avatar,
  Spin,
} from 'antd';
import SearchBar from './fragments/SearchBar';
import { clearMap, onRedirect } from '../../modules/mapFunctions';
import { LocateControl } from './fragments/LocateControl';
import _ from 'lodash';
import config from '../../modules/config';
import MapChange from './MapChange';
import { VehiclePicker } from './fragments/VehiclePicker';
import { ZoomControlCustom } from './fragments/ZoomControlCustom';
import { Link } from 'react-router-dom';
import { utils } from '../../modules/utils';
import authAction from '../../actions/auth';
import RoutingResult from './RoutingResult/RoutingResult';
import TimelineIcon from '@material-ui/icons/Timeline';
import AccessAlarmIcon from '@material-ui/icons/AccessAlarm';
import VOHNews from './VOHNews/VOHNews';

const icon2 = new L.Icon({
  iconUrl: locationIcon,
  iconSize: new L.Point(25, 41),
  className: 'icon-custom',
  // html: <Icon type="plus" />,
});

const icon = new L.Icon({
  iconUrl: locationIcon2,
  iconSize: new L.Point(25, 41),
  className: 'icon-custom',
});

/**
 *
 * @param {array} coords
 */
const locateIcon = (coords) =>
  new L.Icon({
    iconUrl: locateIconImg,
    // iconSize: new L.Point(16, 16),
    iconSize: [16, 16],
    iconAnchor: [10, 10],
    // popupAnchor: [0, -16],
    // className: "icon-custom",
  });

class HomeMap extends React.Component {
  constructor(props) {
    super(props);
    this.mapZoom = 17;
    this.minZoomToGetTraffic = 13;
    this.state = {
      zoom: 17,
      darkmode: false,
      trafficReport: [],
      isRedirect: false,
      redirectType: 'distance',
      positionMark: {},
      countMark: 0,
      positionEndMark: {},
      positionReport: {},
      startResultRedirect: '',
      endResultRedirect: '',
      mapCenter: [10.7725104, 106.6575988],
      locateCoords: [10.7725104, 106.6575988],
      searchCoords: [],
      searchAddress: null,
      isLocation: false,
      mapType: 'basic-map',
      vehicleIndex: 0,
      endSearchValue: '',
      startSearchValue: '',
      isGetReport: true,
      isFirstGetStatus: true,
      showSidebar: true,
      locateChange: 0,
      isSearching: false,
    };
    this.style = this.style.bind(this);
    this.onGetTrafficStatus = _.debounce(this._onGetTrafficStatus, 0, {
      leading: true,
      maxWait: 2000,
    });
  }

  mapRef = createRef();
  map = null;

  componentDidMount() {
    // this.props.changeTrafficStatusField({ "trafficStatus.isCanceled": false });
    this.map = this.mapRef.current.leafletElement;
    const {
      _northEast,
      _southWest,
    } = this.mapRef.current.leafletElement.getBounds();

    navigator.geolocation.getCurrentPosition(
      (success) => {
        const { latitude, longitude } = success.coords;
        this.setState({
          mapCenter: [latitude, longitude],
          locateCoords: [latitude, longitude],
        });
        let query = {
          lat: latitude,
          lng: longitude,
          zoom: this.state.zoom,
          time: new Date().getTime(),
        };
        query = {
          NElat: _northEast.lat,
          NElng: _northEast.lng,
          WSlat: _southWest.lat,
          WSlng: _southWest.lng,
          zoom: this.state.zoom,
        };
        this.onGetTrafficStatus(query, this.state.zoom, true);
      },
      (error) => {
        const { mapCenter, zoom } = this.state;
        let query = {
          lat: mapCenter[0],
          lng: mapCenter[1],
          zoom: zoom,
          time: new Date().getTime(),
        };
        query = {
          NElat: _northEast.lat,
          NElng: _northEast.lng,
          WSlat: _southWest.lat,
          WSlng: _southWest.lng,
          zoom: this.state.zoom,
        };
        this.onGetTrafficStatus(query, this.state.zoom, true);
        // this.setState({ isFirstGetStatus: false });
      }
    );
  }
  componentDidUpdate(prevProps, prevState) {
    console.log(
      'currentBounds',
      this.mapRef.current.leafletElement.getBounds()
    );
    console.log('currentZoom', this.mapRef.current.leafletElement.getZoom());
    console.log(
      'currentCenter',
      this.mapRef.current.leafletElement.getCenter()
    );
    // const { prevMapCenter, prevIsGetReport } = prevState;
    const prevMapCenter = prevState.mapCenter;
    const { mapCenter, isGetReport, zoom, isSearching } = this.state;
    if (
      prevMapCenter[0] !== mapCenter[0] &&
      prevMapCenter[1] !== mapCenter[1] &&
      isGetReport &&
      isSearching
    ) {
      const {
        _northEast,
        _southWest,
      } = this.mapRef.current.leafletElement.getBounds();
      let query = {
        NElat: _northEast.lat,
        NElng: _northEast.lng,
        WSlat: _southWest.lat,
        WSlng: _southWest.lng,
        zoom: zoom,
      };
      this.onGetTrafficStatus(query, zoom, true);
      this.setState({ isSearching: false })
    }
  }
  componentWillReceiveProps(nextProps) {
    const { priority } = nextProps.redirectData;
    const { priority: oldPriority, data } = this.props.redirectData;
    console.log('priority', priority, oldPriority);
    if (priority !== oldPriority) {
      console.log('RECEIVE PROPS RUN');
      const { positionMark, positionEndMark } = this.state;
      onRedirect(
        this.map,
        data,
        {
          zoom: this.state.zoom,
          darkmode: this.state.darkmode,
          slat: positionMark.lat,
          slng: positionMark.lng,
          elat: positionEndMark.lat,
          elng: positionEndMark.lng,
        },
        priority
      );
    }
  }

  componentWillUnmount() {
    this.props.cancelGetTrafficStatus();
    this.props.setTrafficStatusState({ isDrew: false });
  }

  _onGetTrafficStatus(query, zoom, isV2) {
    const { darkmode } = this.state;
    console.log('FirstGetStatus', query);
    if (!zoom) zoom = this.mapZoom;
    this.props
      .getTrafficStatus(this.map, zoom, darkmode, query, isV2)
      .then((success) => {
        this.setState({ isFirstGetStatus: false });
      });
  }

  onChangeMapType = () => {
    if (this.state.mapType === 'basic-map') {
      this.setState({ mapType: 'satellite' });
    } else this.setState({ mapType: 'basic-map' });
  };

  onDragend = (key) => (event) => {
    console.log('DRAGEND MARK', key, event);
    this.setState({ [key]: event.target._latlng });
    const { positionEndMark, positionMark } = this.state;
    if (this.state.isRedirect) {
      if (key === 'positionEndMark') {
        const geocoder = L.Control.Geocoder.nominatim();
        geocoder.reverse(event.target._latlng, this.state.zoom, (res) => {
          if (res && res[0]) {
            let address = res[0].name;
            console.log('ADDRESS WHEN REVERSE VALUE', address);
            this.setState({ endSearchValue: address });
          }
        });

        const queryRedirect = {
          slat: positionMark.lat,
          slng: positionMark.lng,
          elat: event.target._latlng.lat,
          elng: event.target._latlng.lng,
          type: this.state.redirectType,
        };
        this.props.redirect(this.map, queryRedirect);
      } else if (key === 'positionMark') {
        const geocoder = L.Control.Geocoder.nominatim();
        geocoder.reverse(event.target._latlng, this.state.zoom, (res) => {
          let address = res[0].name;
          this.setState({ startSearchValue: address });
        });
        const queryRedirect = {
          slat: event.target._latlng.lat,
          slng: event.target._latlng.lng,
          elat: positionEndMark.lat,
          elng: positionEndMark.lng,
          type: this.state.redirectType,
        };
        this.props.redirect(this.map, queryRedirect);
      }
    }
  };

  addMark = (event) => {
    console.log('Add mark');
    const { isRedirect, countMark, positionMark, redirectType } = this.state;
    if (isRedirect) {
      if (countMark === 0) {
        this.setState({ positionMark: event.latlng, countMark: 1 });
      } else if (countMark === 1) {
        const geocoder = L.Control.Geocoder.nominatim();
        geocoder.reverse(event.latlng, this.state.zoom, (res) => {
          let address = res[0].name;
          this.setState({ endSearchValue: address });
        });
        this.setState({ positionEndMark: event.latlng, countMark: 2 });
        const query = {
          slat: positionMark.lat,
          slng: positionMark.lng,
          elat: event.latlng.lat,
          elng: event.latlng.lng,
          type: redirectType,
        };
        this.props.redirect(this.map, query);
      }
    }
  };

  onViewportChanged = (event) => {
    // const startTime = Date.now();
    const { zoom, center } = event;
    console.log('ON VIEWPORT CHANGE', zoom, center, this.state.isRedirect);
    const [latitude, longitude] = center;
    // this.setState({ mapCenter: [latitude, longitude], zoom });
    // this.mapCenter = [latitude, longitude];
    this.setState({ zoom: zoom, mapCenter: [latitude, longitude] });
    this.mapZoom = zoom;

    if (
      // !this.state.isGetReport ||
      !this.state.isRedirect &&
      zoom < this.minZoomToGetTraffic
    ) {
      clearMap(this.map);
    }

    if (this.state.isRedirect) {
      // const { positionMark, positionEndMark } = this.state;
      const redirectData = this.props.redirectData.data;
      // const coords = {
      //   slat: positionMark.lat,
      //   slng: positionMark.lng,
      //   elat: positionEndMark.lat,
      //   elng: positionEndMark.lng,
      // };
      console.log('REDIRECT DATA', redirectData);
      // onRedirect(
      //   this.map,
      //   redirectData,
      //   coords,
      //   this.props.redirectData.priority
      // );
      return;
    }

    if (!this.state.isFirstGetStatus && this.state.isGetReport) {
      let query = {
        lat: event.center[0],
        lng: event.center[1],
        zoom: zoom,
        time: new Date().getTime(),
      };
      const {
        _northEast,
        _southWest,
      } = this.mapRef.current.leafletElement.getBounds();
      query = {
        NElat: _northEast.lat,
        NElng: _northEast.lng,
        WSlat: _southWest.lat,
        WSlng: _southWest.lng,
        zoom: this.state.zoom,
      };
      this.onGetTrafficStatus(query, zoom, true);
    }
  };

  async toggleGetStatus() {
    console.log('toggle get status');
    if (this.state.isRedirect) {
      // this.setState({ isRedirect: false });
      return;
    }
    let center = this.mapRef.current.leafletElement.getCenter();
    if (!this.state.isGetReport) {
      clearMap(this.map, 'trafficStatus');
      if (this.state.zoom >= this.minZoomToGetTraffic) {
        let query = {
          lat: center.lat,
          lng: center.lng,
          zoom: this.state.zoom,
          time: new Date().getTime(),
        };
        const {
          _northEast,
          _southWest,
        } = this.mapRef.current.leafletElement.getBounds();
        query = {
          NElat: _northEast.lat,
          NElng: _northEast.lng,
          WSlat: _southWest.lat,
          WSlng: _southWest.lng,
          zoom: this.state.zoom,
        };
        this.onGetTrafficStatus(query, this.state.zoom, true);
      }
    } else {
      this.props.cancelGetTrafficStatus();
      clearMap(this.map, 'trafficStatus');
      this.props.setTrafficStatusState({ isDrew: false });
    }

    this.setState({
      isGetReport: !this.state.isGetReport,
    });
  }

  toggleRedirect = () => {
    const { isRedirect } = this.state;
    if (!isRedirect) {
      let center = this.mapRef.current.leafletElement.getCenter();

      const geocoder = L.Control.Geocoder.nominatim();
      geocoder.reverse(center, this.state.zoom, (res) => {
        let address = res[0].name;
        this.setState({ startSearchValue: address });
      });
      this.setState({
        isRedirect: !isRedirect,
        positionMark: center,
        isGetReport: false,
        countMark: 1,
      });
      //abort draw traffic status
      this.props.cancelGetTrafficStatus();
      // this.props.setTrafficStatusState({ isDrew: false });
      clearMap(this.map, 'trafficStatus');
    } else {
      this.setState({
        isRedirect: !isRedirect,
        positionMark: {},
        positionEndMark: {},
        countMark: 0,
        startSearchValue: '',
        endSearchValue: '',
      });
      clearMap(this.map, 'direct');
    }
  };

  style(feature) {
    return {
      opacity: 0.7,
      weight: weightPolyline(this.state.zoom, this.state.darkmode),
      color: gradient(feature.properties.v / 60),
    };
  }

  onLocate = (location) => {
    const { locateChange } = this.state;
    let lat = '';
    let lng = '';
    if (locateChange) {
      console.log('On locate', location);
      const { latitude, longitude } = location.coords;
      this.setState({
        mapCenter: [latitude, longitude],
        locateCoords: [latitude, longitude],
        locateChange: 0,
      });
      lat = latitude;
      lng = longitude;
    } else {
      this.setState({ mapCenter: [10.7725104, 106.6575988], locateChange: 1 });
      [lat, lng] = [10.7725104, 106.6575988];
    }

    if (this.state.isGetReport) {
      const {
        _northEast,
        _southWest,
      } = this.mapRef.current.leafletElement.getBounds();
      let query = {
        lat,
        lng,
        zoom: this.state.zoom,
        time: new Date().getTime(),
      };
      query = {
        NElat: _northEast.lat,
        NElng: _northEast.lng,
        WSlat: _southWest.lat,
        WSlng: _southWest.lng,
        zoom: this.state.zoom,
      };
      this.onGetTrafficStatus(query, this.state.zoom, true);
    }
  };

  onSearch = (result, key) => {
    // const lat = result.y;
    // const lng = result.x;
    const { lat, lng } = result.position;
    console.log('RESULT AND KEY', result, key);
    const {
      positionEndMark,
      positionMark,
      isRedirect,
      redirectType,
      // isGetReport,
      // zoom,
    } = this.state;
    if (!isRedirect) {
      this.setState({
        mapCenter: [lat, lng],
        searchCoords: [lat, lng],
        searchAddress: result.label,
        isSearching: true,
      });
    } else {
      let query = {};
      if (key === 'value') {
        this.setState({
          mapCenter: [lat, lng],
          positionMark: { lat, lng },
          countMark: 1,
        });
        if (positionEndMark.lat) {
          query = {
            slat: lat,
            slng: lng,
            elat: positionEndMark.lat,
            elng: positionEndMark.lng,
            type: redirectType,
          };
        }
      } else if (key === 'endValue') {
        this.setState({
          mapCenter: [lat, lng],
          positionEndMark: { lat, lng },
          countMark: 2,
        });
        if (positionMark.lat) {
          query = {
            slat: positionMark.lat,
            slng: positionMark.lng,
            elat: lat,
            elng: lng,
            type: redirectType,
          };
        }
      }
      if (!_.isEmpty(query)) {
        this.props.redirect(this.map, query);
      }
    }
  };

  onSelectVehicle = (data, index) => {
    if (data === 'close') {
      this.toggleRedirect();
    } else {
      const { positionEndMark, positionMark } = this.state;
      const query = {
        slat: positionMark.lat,
        slng: positionMark.lng,
        elat: positionEndMark.lat,
        elng: positionEndMark.lng,
        type: data.value,
      };
      this.setState({ redirectType: data.value, vehicleIndex: index });
      this.props.redirect(this.map, query);
    }
  };

  onSelectRedirectType = (value) => {
    this.setState({ redirectType: value[0] });
    const { positionEndMark, positionMark } = this.state;
    const query = {
      slat: positionMark.lat,
      slng: positionMark.lng,
      elat: positionEndMark.lat,
      elng: positionEndMark.lng,
      type: value[0],
    };
    this.props.redirect(this.map, query);
  };

  handleReverseRedirect = () => {
    const { positionEndMark, positionMark, redirectType, startSearchValue, endSearchValue } = this.state;
    if (!positionEndMark.hasOwnProperty('lat')) {
      return;
    }

    const query = {
      slat: positionEndMark.lat,
      slng: positionEndMark.lng,
      elat: positionMark.lat,
      elng: positionMark.lng,
      type: redirectType,
    };

    this.setState({
      positionMark: positionEndMark,
      positionEndMark: positionMark,
      startSearchValue: endSearchValue,
      endSearchValue: startSearchValue,
    })
    this.props.redirect(this.map, query);
  }

  /**
   *
   * @param {'plus'|'minus'} value
   */
  onZoomControlChange = (value) => {
    if (value === 'minus') {
      if (this.state.zoom === 5) return;
      this.setState({ zoom: this.state.zoom - 1 });
    } else {
      if (this.state.zoom === 19) return;
      this.setState({ zoom: this.state.zoom + 1 });
    }
  };

  toggleSidebar = () => {
    console.log('Toggle sidebar');
    this.setState((state) => {
      console.log('Toggle sidebar', state);
      return { showSidebar: !state.showSidebar };
    });
  };

  renderSidebar() {
    const {
      isRedirect,
      isGetReport,
      startSearchValue,
      endSearchValue,
      redirectType,
    } = this.state;

    const { redirectData, isLogin, user } = this.props;
    const searchRender = !isRedirect ? (
      <SearchBar
        onSearch={(result, key) => this.onSearch(result, key)}
        isRedirect={isRedirect}
        showDirectButton={true}
        toggleRedirect={this.toggleRedirect}
      />
    ) : (
      <Row>
        <Col span={24}>
          <VehiclePicker
            // onCloseRedirect={this.isRedirect}
            data={
              [
                // { value: "time", text: <FaClock className="vehicle-icon" /> },
                // { value: "distance", text: <FaRoad className="vehicle-icon" /> },
              ]
            }
            onSelect={this.onSelectVehicle}
          />
        </Col>
        <Col span={24}>
          <SearchBar
            onSearch={(result, key) => this.onSearch(result, key)}
            isRedirect={isRedirect}
            value={startSearchValue}
            endValue={endSearchValue}
            handleReverseRedirect={this.handleReverseRedirect}
            setTextValue={(startValue, endValue) => {
              this.setState({
                startSearchValue: startValue,
                endSearchValue: endValue
              })
            }}
          />
        </Col>
        <div style={{ paddingRight: '32px', paddingLeft: '3px', display: 'flex', alignItems: 'center' }}>
          {
            this.state.redirectType === 'distance' ? (
              <TimelineIcon style={{ marginRight: '5px', color: 'rgb(160, 217, 245)' }} />
            ) : (
              <AccessAlarmIcon style={{ marginRight: '9px', color: 'rgb(160, 217, 245)', fontSize: '20px' }} />
            )
          }
          <Cascader
            value={[this.state.redirectType]}
            options={[
              { value: 'distance', label: 'Tìm theo khoảng cách ngắn nhất' },
              { value: 'time', label: 'Tìm theo thời gian ngắn nhất' },
            ]}
            onChange={this.onSelectRedirectType}
            style={{ width: '100%' }}
            suffixIcon={redirectData.isFetching && <Icon type='loading' />}
            allowClear={false}
          />
        </div>
      </Row>
    );

    return (
      <div>
        <div
          className='sidebar-depart py-2 border-bottom'
          style={{
            background: isRedirect && '#1A73E8',
          }}
        >
          {searchRender}
        </div>
        {isRedirect && <RoutingResult redirectType={redirectType} data={redirectData.data} />}
        <div className='mt-2 py-3 sidebar-depart border-top'>
          <Row>
            <div style={{ fontSize: '1.3em', fontWeight: 'bold' }}>
              Cảnh báo tình trạng giao thông
            </div>
          </Row>
          <Row justify='start' className='mt-3'>
            <Link to='/user'>
                <Col span={3} className='mt-1'>
                  <div className="icon-button form">
                    <i className="fa fa-file mr-2 icon-form"></i>
                    <span></span>
                  </div>
                </Col>
                <Col span={20} className='ml-2'> 
                  <Row>
                    <div className='title'>Biểu mẫu</div>
                    <div className='description'>Điền vào biểu mẫu có sẵn</div>
                  </Row>
                </Col>
            </Link>
          </Row>
          <hr className="solid"></hr>
          <Row justify='start' className='mb-3'>
            <Link to='/speech-report'>
                <Col span={3} className='mt-1'>
                  <div className="icon-button mic">
                    <i className="fa fa-microphone mr-2 icon-mic"></i>
                    <span></span>
                  </div>
                </Col>
                <Col span={18} className='ml-2'> 
                  <Row>
                    <div className='title'>Giọng nói</div>
                    <div className='description'>Gửi tập tin ghi âm thanh</div>
                  </Row>
                </Col>
            </Link>
          </Row>
          <Row>
            <Col span={20}>
              <div style={{ fontSize: '1em', fontWeight: 'bold' }}>
                Xem tình trạng giao thông
              </div>
            </Col>
            <Col span={4}>
              <Switch
                style={{
                  border: 'solid 1px #fff',
                  backgroundColor: isGetReport ? '' : '#ccc',
                }}
                onChange={() => this.toggleGetStatus()}
                checked={isGetReport}
              // loading={statusFetching}
              />
            </Col>
          </Row>
        </div>
        {isLogin ? (
          <div className='w-100 mt-2 py-3 sidebar-depart border-top border-bottom'>
            <Row>
              <Col span={6}>
                <Row type='flex' justify='center'>
                  <Avatar
                    size={56}
                    alt={'avatar'}
                    src={user.avatar || require('../../static/images/user.png')}
                  />
                </Row>
              </Col>

              <Col span={18}>
                <div style={{ fontWeight: 'bold' }}>
                  {user.name || user.username}
                </div>
                <div className='user-info-item'>
                  Uy tín:{' '}
                  {user.reputation ? Math.round(user.reputation * 100) : 0}/100
                </div>
                {user.phone ? (
                  <div className='user-info-item'>
                    Số điện thoại: {user.phone}
                  </div>
                ) : null}
                {user.email ? (
                  <div className='user-info-item'>Email: {user.email}</div>
                ) : null}
                <div className='user-info-item'>
                  Ngày tham gia:{' '}
                  {utils.convertTimeToString(user.createdAt, 'DD/MM/YYYY')}
                </div>
                <Row type='flex' justify='end'>
                  <div
                    style={{
                      cursor: 'pointer',
                      fontSize: '12px',
                      border: '1px solid gray',
                      borderRadius: '8px',
                      padding: '4px 8px',
                    }}
                    onClick={this.props.logout}
                    className='d-inline-block'
                  >
                    <i className='fa fa-sign-out mr-2' aria-hidden='true'></i>
                    Đăng xuất
                  </div>
                </Row>
              </Col>
            </Row>
          </div>
        ) : (
          <div className='w-100 mt-2 py-3 sidebar-depart border-top border-bottom'>
            <p className='text-center'>
              Đăng nhập hoặc đăng ký ngay để trải nghiệm đầy đủ tính năng của BK
              Traffic.
            </p>
            <Row type='flex' justify='center' className='mb-3'>
              <Link to='/login' className='mx-2'>
                <Button>
                  <i
                    className='fa fa-sign-in mr-2'
                    aria-hidden='true'
                    style={{ color: 'blue' }}
                  ></i>
                  Đăng nhập
                </Button>
              </Link>
              <Link to='/signup' className='mx-2'>
                <Button>
                  <i
                    className='fa fa-user-plus mr-2'
                    aria-hidden='true'
                    style={{ color: 'blue' }}
                  ></i>
                  Đăng ký
                </Button>
              </Link>
            </Row>
          </div>
        )}
        <VOHNews />
      </div>
    );
  }

  render() {
    const {
      isRedirect,
      positionMark,
      positionEndMark,
      mapCenter,
      locateCoords,
      mapType,
      searchCoords,
      // zoom,
    } = this.state;
    console.log('MAP CENTER', mapCenter, locateCoords);

    const { redirectData } = this.props;
    const { isFetching: statusFetching } = this.props.trafficStatus;
    const startTime = Date.now();
    let tileUrl = config.MAP_URL;
    if (mapType === 'satellite') {
      tileUrl = config.SATELLITE_URL;
    }

    const markers = [
      <Marker
        key={'myLocation'}
        position={locateCoords}
        draggable={false}
        icon={locateIcon(locateCoords)}
      />,
    ];
    console.log(isRedirect);
    if (isRedirect) {
      if (this.state.zoom > 13) {
        if (positionMark && positionMark.lat && positionMark.lng) {
          markers.push(
            <Marker
              key={'startMark'}
              position={positionMark}
              draggable={true}
              onDragend={this.onDragend('positionMark')}
              icon={icon2}
              id='beginMark'
            >
              <Tooltip>Điểm đầu</Tooltip>
            </Marker>
          );
        }
        if (positionEndMark && positionEndMark.lat && positionEndMark.lng) {
          markers.push(
            <Marker
              key='endMark'
              position={positionEndMark}
              draggable={true}
              onDragend={this.onDragend('positionEndMark')}
              icon={icon}
              id='endMark'
            >
              <Tooltip>Điểm cuối</Tooltip>
            </Marker>
          );
        }
      }
    } else if (searchCoords && searchCoords.length > 0) {
      markers.push(
        <Marker position={searchCoords} draggable={false} key={'search'} />
      );
    }

    const mainView = (
      <Row style={{ height: '100%' }}>
        <Col
          style={{
            zIndex: 1,
            // height: "100%",
          }}
          className='position-absolute'
        >
          <div className='row'>
            <div
              style={{
                width: '29em',
                display: this.state.showSidebar ? 'block' : 'none',
                borderTopRightRadius: '4px',
                borderBottomRightRadius: '8px',
                overflowX: 'hidden',
                overflowY: 'scroll',
              }}
              className='sidebar'
            >
              {this.renderSidebar()}
            </div>
            <AntTooltip
              title={
                this.state.showSidebar
                  ? 'Ẩn bảng điều khiển'
                  : 'Hiện bảng điều khiển'
              }
              placement='topRight'
            >
              <div
                style={{
                  marginTop: '1em',
                  opacity: 0.9,
                  width: '1.2em',
                  height: '3em',
                  background: 'white',
                  borderTopRightRadius: '4px',
                  borderBottomRightRadius: '4px',
                  zIndex: 2,
                }}
                className='row justify-content-center align-items-center toggle-sidebar-btn'
                onClick={this.toggleSidebar}
              >
                {this.state.showSidebar ? (
                  <i className='fa fa-chevron-left' aria-hidden='true'></i>
                ) : (
                  <i className='fa fa-chevron-right' aria-hidden='true'></i>
                )}
              </div>
            </AntTooltip>
          </div>
        </Col>
        <Col span={24} className={this.state.showSidebar ? 'map-container show-sidebar' : 'map-container'}>
          <div style={{ height: '100%', width: '100%' }}>
            <div className='leaflet-map' style={{ height: '100%' }}>
              <div className='map-bottom-right-container'>
                <div className='row justify-content-center'>
                  <LocateControl onLocate={this.onLocate} />
                </div>
                <div className='row justify-content-center'>
                  <ZoomControlCustom
                    onChange={this.onZoomControlChange}
                    max={21}
                    min={5}
                  />
                </div>
                {/* <ZoomControl /> */}

                <div
                  className='row justify-content-center mt-2 px-1 py-2'
                  style={{
                    opacity: statusFetching || redirectData.isFetching ? 1 : 0,
                    backgroundColor: 'rgba(0,0,0,0.4)',
                    borderRadius: '16px',
                  }}
                >
                  <Spin></Spin>
                  <div className='w-100 text-center'>Loading</div>
                </div>
              </div>

              <Map
                zoom={this.state.zoom}
                style={{ height: '100%' }}
                ref={this.mapRef}
                maxZoom={19}
                minZoom={5}
                scrollWheelZoom={true}
                zoomControl={false}
                zoomAnimation={true}
                onClick={this.addMark}
                // show
                onViewportChanged={this.onViewportChanged}
                center={mapCenter}
              // useFlyTo={true}
              >
                <>
                  <MapChange
                    center={mapCenter}
                    mapType={mapType}
                    onChange={this.onChangeMapType}
                  />
                  <TileLayer url={tileUrl} detectRetina={true} />
                  {/* <Marker position={locateCoords} /> */}
                  {markers}
                </>
              </Map>
            </div>
          </div>
        </Col>
      </Row>
    );
    console.log('Render map in', Date.now() - startTime);

    console.log('redirectData', redirectData);

    return mainView;
  }
}

const mapStateToProps = (state) => ({
  redirectData: state.redirect,
  trafficStatus: state.trafficStatus.trafficStatus,
  user: state.user.userInfo.user,
  isLogin: state.app.isLogin,
});

const mapDispatchToProps = {
  getTrafficStatus: trafficStatusAction.getTrafficStatus,
  redirect: redirectAction.redirect,
  changeTrafficStatusField: trafficStatusAction.changeField,
  cancelGetTrafficStatus: trafficStatusAction.cancelGetTrafficStatus,
  setTrafficStatusState: trafficStatusAction.changeField,
  logout: authAction.logout,
};

export default connect(mapStateToProps, mapDispatchToProps)(HomeMap);
