import React, { useMemo, useState, useEffect } from 'react';
import PropTypes from 'prop-types';
import { useSelector } from 'react-redux';

import './RoutingResult.css';
import RoutingResultItem from './RoutingResultItem';

// Value of currentRoute is 0 or 1
const RoutingResult = props => {
  const { data, redirectType } = props;
  const [currentRoute, setCurrentRoute] = useState(0);

  const currentRouteId = useSelector((state) => state.vohNews.currentRouteId);

  const dataToShow = useMemo(() => {
    if (redirectType === 'distance') {
      return [...data.sort((a, b) => a.distance - b.distance)];
    }
    return [...data.sort((a, b) => a.time - b.time)]
  }, [data, redirectType]);

  useEffect(() => {
    setCurrentRoute(0);
  }, [dataToShow]);

  useEffect(() => {
    const currentRouteDataIndex = dataToShow.findIndex((item) => item._id === currentRouteId);
    if (currentRouteDataIndex !== -1) {
      setCurrentRoute(currentRouteDataIndex);
    }
  }, [currentRouteId, dataToShow]);

  return (
    <div className="routing-result-wrapper">
      {
        dataToShow.map((item, index) => {
          const firstStreet = item.coords.find(item => item.street.name !== 'Không xác định');
          return (
            <RoutingResultItem redirectType={redirectType}
            key={item._id} data={item} index={index}
            firstStreetName={firstStreet?.street?.name}
            currentRoute={currentRoute}
            setCurrentRoute={setCurrentRoute} />
          )
        })
      }
    </div>
  )
}

RoutingResult.propTypes = {
  data: PropTypes.array,
  redirectType: PropTypes.string,
}

RoutingResult.defaultProps = {
  data: [],
  redirectType: 'distance',
}

export default RoutingResult
