import React, { useMemo } from 'react';
import PropTypes from 'prop-types';
import { Grid } from '@material-ui/core';
import { makeStyles } from '@material-ui/core/styles';
import DriveEtaIcon from '@material-ui/icons/DriveEta';
import { useDispatch } from 'react-redux';

import { around } from '../../../functions/common';
import './RoutingResult.css';
import RedirectAction from '../../../actions/redirect';

const useStyles = makeStyles({
  root: {
    borderBottom: '1px solid #e6e6e6',
    padding: '16px 24px 16px 16px',
    cursor: 'pointer',
    '&:hover': {
      backgroundColor: '#f5f4f4'
    },
    backgroundColor: props => props.isActive && '#f5f4f4',
  }
})

const RoutingResultItem = props => {
  const dispatch = useDispatch();
  const { data, index, redirectType, firstStreetName, currentRoute, setCurrentRoute } = props;
  const { time, distance, coords } = data;
  
  const classes = useStyles({ isActive: index === currentRoute});
  
  const streetName = useMemo(() => {
    if (index === 0) {
      return coords[0].street.name;
    }
    const streetFound = coords.find(item => item.street.name !== firstStreetName && 
      item.street.name !== 'Không xác định');
    return streetFound.street.name;
  }, [coords, index, firstStreetName]);

  const handleOnClickItem = () => {
    if (currentRoute !== index) {
      setCurrentRoute(index);
      dispatch(RedirectAction.setState({ priority: 1 }));
    }
  }

  return (
    <Grid container className={classes.root} onClick={handleOnClickItem}>
      <Grid item xs={2}>
        <DriveEtaIcon />
      </Grid>
      <Grid item xs={7}>
        <h6>Qua {streetName}</h6>
        {index === 0 && <p>Tuyến đường {redirectType === 'distance' ? 'ngắn' : 'nhanh'} nhất</p>}
      </Grid>
      {
        redirectType === 'distance' ? (
          <Grid item xs={3}>
            <h6 className="routing-result-item-right">{around(distance/1000, 1)} km</h6>
            <p className="routing-result-item-right">{Math.ceil(time)} phút</p>
          </Grid>
        ) : (
          <Grid item xs={3}>
            <h6 className="routing-result-item-right">{Math.ceil(time)} phút</h6>
            <p className="routing-result-item-right">{around(distance/1000, 1)} km</p>
          </Grid>
        )
      }
    </Grid>
  )
}

RoutingResultItem.propTypes = {
  data: PropTypes.object,
  index: PropTypes.number,
  redirectType: PropTypes.string,
  firstStreetName: PropTypes.string,
  currentRoute: PropTypes.number,
  setCurrentRoute: PropTypes.func,
}

RoutingResultItem.defaultProps = {
  data: {},
  index: 0,
  redirectType: 'distance',
  firstStreetName: '',
  currentRoute: 0,
  setCurrentRoute: () => {},
}

export default RoutingResultItem
