import L from "leaflet";
import { store } from "../../../App";
import redirect from "../../../actions/redirect";
import trafficStatus from "../../../actions/traffic-status";
import { VOHNewsActions } from '../../../actions/VOHNews';

function d2h(d) {
  var hex = "0123456789ABCDEF";
  var r = "";
  d = Math.floor(d);
  while (d !== 0) {
    r = hex[d % 16] + r;
    d = Math.floor(d / 16);
  }
  while (r.length < 2) r = "0" + r;
  return r;
}

export function gradient(color) {
  function f2h(d) {
    if (d < 1) {
      return d2h(256 * d);
    } else {
      return d2h(256 * d - 1)
    }   
  }

  if (color < 0) {
    return "#FF0000"; // darkred #FF0000
  }
  if (color < 1.0 / 3) {
    return "#FF" + f2h(3 * color) + "00"; // darkred #FF0000 to yellow #FFFF00
  }
  if (color < 2.0 / 3) {
    return "#" + f2h(2 - 3 * color) + "FF00"; // yellow #FFFF00 to green #00FF00
  }
  if (color < 1) {
    return "#00FF" + f2h(3 * color - 2); // green #00FF00 to blue #00FFFF
  }
  return "#00FFFF";
}

export function weightPolyline(zoom, darkmode) {
  var width;
  switch (zoom) {
    case 21:
      width = 7;
      break;
    case 20:
      width = 6;
      break;
    case 19:
      width = 5;
      break;
    case 18:
      width = 4;
      break;
    case 17:
      width = 3;
      break;
    case 16:
      width = 2;
      break;
    case 15:
      width = 2;
      break;
    case 14:
      width = 1;
      break;
    case 13:
      width = 1;
      break;
    case 12:
      width = 1;
      break;
    default:
      width = 1;
  }
  if (darkmode) {
    return width;
  } else {
    return width * 1.3;
  }
}

function highlightFeature(e) {
  var map = this._map;
  var layer = e.target;
  layer.setStyle({
    weight: layer.options.weight * 1.2,
    // weight: 7,
    opacity: 1,
  });
  layer.bringToBack();
  const { v, source } = layer.feature.properties;
  const velocity = Math.round(v);
  L.popup()
    .setLatLng(e.latlng)
    .setContent("Vận tốc: " + velocity + " km/h </br> Nguồn: " + source)
    .openOn(map);
  // setTimeout(function () {
  //   map.closePopup(popupPosition);
  // }, 3000);
}

function highlightDistance(e) {
  var map = this._map;
  var layer = e.target;
  // layer.setStyle({
  //   weight: 7,
  //   opacity: 1,
  //   width: 30,
  // });
  layer.bringToBack();
  const { distance, time } = layer.feature.properties;
  const hour = Math.floor(time / 60);
  const minutes = Math.ceil(time % 60);
  L.popup()
    .setLatLng(e.latlng)
    .setContent(
      `Khoảng cách: ${
        distance < 1000 ? distance + " m" : distance / 1000 + " km"
      }  <br/> Thời gian di chuyển: ${hour > 0 ? hour + "h" : ""}${minutes}'`
    )
    .openOn(map);
  // setTimeout(function () {
  //   map.closePopup(popupPosition);
  // }, 3000);
}

function highlightReportDistance(e) {
  var map = this._map;
  var layer = e.target;
  layer.bringToBack();

  const { distance } = layer.feature.properties;
  L.popup()
    .setLatLng(e.latlng)
    .setContent(
      `Khoảng cách: ${
        distance < 1000 ? distance + " m" : distance / 1000 + " km"
      }`
    )
    .openOn(map);
}


const clickDistance = (e) => {
  console.log("ON cLICK");
  // var map = this._map;
  var layer = e.target;
  layer.setStyle({
    // weight: layer.options.weight * 1.2,
    weight: 7,
    opacity: 1,
    width: 30,
  });
  
  layer.bringToBack();
  const { index, _id } = layer.feature.properties;
  const priority = store.getState().redirect.priority;

  store.dispatch(VOHNewsActions.setIdOfCurrentRoute(_id));

  if (priority !== index) {
    store.dispatch(redirect.setState({ priority: index }));
    // onRedirect();
  }
};

const menuDistanceReport =(e)=> {
  store.dispatch(trafficStatus.changeField({
    'reportMap.openedPopover': e.sourceTarget.index
  }))
}

function resetHighLight(e) {
  // var layer = e.target;
  // layer.setStyle({
  //   weight: 7,
  //   opacity: 1,
  // });
  this._map.closePopup();
}
function resetFeatureHighlight(e) {
  e.target.setStyle({
    weight: e.target.options.weight / 1.2,
    opacity: 1,
  });
  this._map.closePopup();
}

export function onEachFeature(feature, layer) {
  layer.on({
    click: highlightFeature,
    mouseover: highlightFeature,
    mouseout: resetFeatureHighlight,
  });
  layer.tag = "GeoJSON";
  layer.type = "trafficStatus";
}

export const onEachDistance = (map) => (feature, layer) => {
  layer.on({
    click: clickDistance,
    mouseover: highlightDistance,
    mouseout: resetHighLight,
  });
  // console.log("FEATURE", feature);
  // if (feature.properties) {
  //   const { distance, lat, lng, isMiddle } = feature.properties;
  //   if (isMiddle && map) {
  //     L.popup().setLatLng({ lat, lng }).setContent(`${distance}`).openOn(map);
  //   }
  // }
  layer.tag = "GeoJSON";
  layer.type = "direct";
};

export const onEachReportDistance = (map) => (feature, layer) => {
  // this data is before update, when onEachReportDistance success, the length of multiReportData increase by 1
  const multiReportData = store.getState().trafficStatus.report.multiReportData;
  layer.on({
    contextmenu: menuDistanceReport,
    mouseover: highlightReportDistance,
    mouseout: resetHighLight,
  });
  layer.tag = "GeoJSON";
  layer.type = "report-direct";
  layer.index = multiReportData.length > 0 ? multiReportData[multiReportData.length - 1].id : 0
};

export function changeMapStyle(mode) {}
