import React from 'react'
import { Form, Icon, Button } from 'semantic-ui-react'
import { timeToFrame, dayToWeekend, timetoPeakedTime } from '../../../functions/converterTime';

const dayOptions = [
    { key: 'mon', value: 0, text: 'Thứ Hai'},
    { key: 'tue', value: 1, text: 'Thứ Ba'},
    { key: 'wed', value: 2, text: 'Thứ Tư'},
    { key: 'thu', value: 3, text: 'Thứ Năm'},
    { key: 'fri', value: 4, text: 'Thứ Sáu'},
    { key: 'sat', value: 5, text: 'Thứ Bảy'},
    { key: 'sun', value: 6, text: 'Chủ Nhật'}   
]

export default class PredictTime extends React.Component {
    constructor(props) {
        super(props);
        this.day = NaN;
        this.hour = NaN;
        this.minute = NaN;
    }

    handleOnChangeDay = (e, {value}) => {
        this.day = value;
    }

    handleOnChangeHour = e => {
        let value = parseInt(e.target.value);
        value = (value < 0) ? 0 : ((value > 23) ? 23 : value);
        e.target.value = value;
        this.hour = value;
    }

    handleOnChangeMinute = e => {
        let value = parseInt(e.target.value);
        value = (value < 0) ? 0 : ((value > 59) ? 59 : value);
        e.target.value = value;
        this.minute = value;
    }

    handleClick = () => {
        let predictTime;
        if (isNaN(this.day) || isNaN(this.hour)) {
            predictTime = -1;
        } else {
            if (isNaN(this.minute)) {
                this.minute = 0;
            }
            predictTime = {
                frame       : timeToFrame(this.hour, this.minute),
                weekday     : this.day,
                isPeakedTime: timetoPeakedTime(this.hour),
                isWeekend   : dayToWeekend(this.day)
            }
        }
        this.props.handleGetPredictTime(predictTime);
    }

    render() {

        return(
            <Form className="predict-time">
                <Form.Group>
                    <Form.Select placeholder='Chọn ngày' options={dayOptions} onChange={this.handleOnChangeDay}/>
                    <Form.Input placeholder='Giờ' type='number' onChange={this.handleOnChangeHour}/>
                    <Form.Input placeholder='Phút' type='number' onChange={this.handleOnChangeMinute}/>
                    <Button type='submit' onClick={this.handleClick}>
                            <Icon name='arrow right'/>
                    </Button>
                </Form.Group>
            </Form>
        )
    }
}