import React from "react";
import { OpenStreetMapProvider } from "leaflet-geosearch";
import { Input, Icon, Row, Col, Tooltip as AntTooltip, Button } from "antd";
import "./fragments-style.css";
import { FaSearch } from "react-icons/fa";
import _ from "lodash";
import { connect } from "react-redux";
import { hereMapActions } from "../../../actions/here-map";
import LocationOnIcon from '@material-ui/icons/LocationOn';
import MoreVertIcon from '@material-ui/icons/MoreVert';
import MyLocationIcon from '@material-ui/icons/MyLocation';
import { Grid } from '@material-ui/core';
import SwapVertIcon from '@material-ui/icons/SwapVert';

class RawSearchBar extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      value: "",
      results: [],
      endValue: "",
      keyChange: "value",
    };
    this._onSearch = _.debounce(this.onSearch, 500);
  }

  componentWillReceiveProps({ value, endValue }) {
    if (value && value !== this.props.value) {
      this.setState({ value });
    }
    if (endValue && endValue !== this.props.endValue) {
      this.setState({ endValue });
    }
  }

  provider = new OpenStreetMapProvider();

  onInput = (event, key = "value") => {
    const value = event.target.value;
    this.setState({ [key]: value, keyChange: key });
    this._onSearch(value);
    // this.onSearch(event.target.value);
  };

  onSearch = async (value) => {
    if (value) {
      this.props.search(value);
    }
  };

  onChose = (result) => {
    if (result) {
      this.setState({ [this.state.keyChange]: result.label });
      this.props.onChangeField("data", []);
      if (this.props.onSearch)
        this.props.onSearch(result, this.state.keyChange);
      if (this.props.setTextValue) {
        setTimeout(() => {
          this.props.setTextValue(this.state.value, this.state.endValue);
        }, 0);
      }
    }
  };

  onKeyDown = (event) => {
    if (event.key === "Enter") {
      this.onChose(this.props.hereMap.data[0]);

      // this.setState({ results: [] });
      this.onSearch("Đại học bách khoa");
    }
  };

  render() {
    const { keyChange } = this.state;
    const { isRedirect, placeholder, hereMap, showDirectButton } = this.props;
    const { data: results } = hereMap;
    const addressItems = results
      .filter((item) => item.label && item.label.includes("Việt Nam"))
      .map((item, index) => {
        return (
          <div
            className="search-item ellipsis-text"
            onClick={() => {
              console.log("INDEX CHOSE", index);
              this.onChose(item);
            }}
            key={index}
          >
            <Grid container alignItems="center" spacing={1}>
              <Grid item xs={1}>
                <LocationOnIcon style={{ color: '#859ba6' }} />
              </Grid>
              <Grid item xs={11} style={{ whiteSpace: 'break-spaces' }}>
                {item.label}
              </Grid>
            </Grid>
          </div>
        );
      });
    let mainView;
    if (isRedirect) {
      mainView = (
        <div style={{ position: 'relative', paddingRight: '28px' }}>
          <MoreVertIcon style={{ color: 'white', position: 'absolute', top: '38px', fontSize: '22px', left: '1px' }} />
          <SwapVertIcon onClick={this.props.handleReverseRedirect}
          style={{ color: 'white', position: 'absolute', top: '38px', fontSize: '32px', right: '-4px', cursor: 'pointer' }} />
          <Grid container alignItems="center">
            <Grid item xs={1}>
              <MyLocationIcon style={{ color: '#80ff01', fontSize: '20px', marginLeft: '2px' }} />
            </Grid>
            <Grid item xs={11}>
              <Input
                value={this.state.value}
                onChange={(event) => this.onInput(event)}
                onKeyDown={(event) => this.onKeyDown(event)}
                placeholder={placeholder || "Vị trí hiện tại"}
                addonAfter={
                  hereMap.isFetching && keyChange === "value" ? (
                    <Icon type="loading" />
                  ) : (
                    <FaSearch />
                  )
                }
                // onClick={() => this.onFocus("isFirstSearchFocus")}
                // onBlur={() => this.onBlur("isFirstSearchFocus")}
                className={"search-bar-input "}
              />
            </Grid>
          </Grid>
          <Grid container alignItems="center">
            <Grid item xs={1}>
              <LocationOnIcon style={{ color: 'red' }} />
            </Grid>
            <Grid item xs={11}>
              <Input
                value={this.state.endValue}
                onChange={(event) => this.onInput(event, "endValue")}
                onKeyDown={(event) => this.onKeyDown(event)}
                placeholder={"Tìm điểm đích hoặc nhấp trên bản đồ"}
                addonAfter={
                  hereMap.isFetching && keyChange === "endValue" ? (
                    <Icon type="loading" />
                  ) : (
                    <FaSearch />
                  )
                }
                // onClick={() => this.onFocus("isSecondSearchFocus")}
                // onBlur={() => this.onBlur("isSecondSearchFocus")}
                className={"search-bar-input mt-1 "}
              />
            </Grid>
          </Grid>
        </div>
      );
    } else {
      if (showDirectButton) {
        mainView = (
          <Row type="flex">
            <Col span={20}>
              <Input
                value={this.state.value}
                onChange={(event) => this.onInput(event)}
                onKeyDown={(event) => this.onKeyDown(event)}
                placeholder={placeholder || "Tìm kiếm trên bản đồ"}
                addonAfter={
                  hereMap.isFetching ? <Icon type="loading" /> : <FaSearch />
                }
                // onFocus={() => this.onFocus("isFirstSearchFocus")}
                // onBlur={() => this.onBlur("isFirstSearchFocus")}
                className={"search-bar-input "}
              />
            </Col>
            <Col span={4} style={{ paddingLeft: "3px" }}>
              <AntTooltip placement="bottom" title="Tìm đường đi">
                <Button
                  style={{
                    display: "flex",
                    width: "100%",
                    height: "100%",
                    justifyContent: "center",
                    alignItems: "center",
                  }}
                  onClick={this.props.toggleRedirect}
                >
                  <img
                    src={require("../../../static/images/turn-right.png")}
                    alt=""
                    style={{
                      width: "20px",
                      height: "20px",
                    }}
                  />
                </Button>
              </AntTooltip>
            </Col>
          </Row>
        );
      } else {
        mainView = (
          <Row type="flex">
            <Col span={24}>
              <Input
                value={this.state.value}
                onChange={(event) => this.onInput(event)}
                onKeyDown={(event) => this.onKeyDown(event)}
                placeholder={placeholder || "Tìm kiếm trên bản đồ"}
                addonAfter={
                  hereMap.isFetching ? <Icon type="loading" /> : <FaSearch />
                }
                // onFocus={() => this.onFocus("isFirstSearchFocus")}
                // onBlur={() => this.onBlur("isFirstSearchFocus")}
                className={"search-bar-input "}
              />
            </Col>
          </Row>
        );
      }
    }
    return (
      <div style={{ position: "relative" }}>
        {mainView}
        <div
          // className="shadow search-result-container"
          style={this.props.recommendStyle}
        >
          <div style={{ paddingLeft: isRedirect && '30px' }}>
            {addressItems}
          </div>
        </div>
      </div>
    );
  }
}

const mapStateToProps = (state) => ({
  hereMap: state.hereMap,
});
const mapDispatchToProps = {
  search: hereMapActions.search,
  onChangeField: hereMapActions.changeField,
};

export default connect(mapStateToProps, mapDispatchToProps)(RawSearchBar);
/**
 * @typedef {'isFirstSearchFocus' | 'isSecondSearchFocus'} SearchFocusType
 */
