import React from "react";
import { Row, Col, Container } from "reactstrap";
import { connect } from "react-redux";
import { weatherAction } from "../../../actions/weather";
const baseImage = "https://openweathermap.org/img/wn/";

const weatherIconMap = {
  'Clear': '01',
  'Clouds': '03',
  'Drizzle': '09',
  'Rain': '10',
  'Thunderstorm': '11',
  'Mist': '50'
};

class WeatherPopup extends React.Component {

  constructor(props) {
    super();
    this.state = {
      loaded: false,
      weatherData: {}
    };
  }

  componentDidMount() {
    this.props.getWeatherInfo().then(res => {
      this.setState({ loaded: true, weatherData: res});
    });
  }

  render() {
    let { loaded, weatherData } = this.state;
    let hour = new Date().getHours();
    let isDay = hour < 18 && hour > 5;
    let icon = weatherIconMap[weatherData.mainStatus];
    let isDayText = isDay ? "d" : "n";
    let isDayColor = isDay ? "gold, orange)" : "skyblue, mediumblue)";
    let backgroundTheme = "linear-gradient(to bottom right, " + isDayColor;
    let image = `${baseImage}${icon}${isDayText}@4x.png`;
    return loaded? (
        <Container>
          <Row className="weather-popup" style={{background: backgroundTheme}}>
            <Col xs="6" className="Weather padding-0 px-0 g-0">
              <div className="weather-item">
                <img 
                  src={image} 
                  alt="Weather Icon"
                ></img>
              </div>
            </Col>
            <Col xs="6" className="padding-0 px-0 g-0">
               <div className="weather-item" style={{color: "white", fontSize: "42px", fontWeight: "bold"}}>
                {Math.round(weatherData.temperature)}ᵒ
              </div>
            </Col>
          </Row>
        </Container>
    ) : <span></span>;
  }
}

const mapStateToProps = (state) => ({
  weatherData: state.weatherData
})

const mapDispatchToProps = {
  getWeatherInfo: weatherAction.getWeatherInfo
}

export default connect(mapStateToProps, mapDispatchToProps)(WeatherPopup);