import React from "react";
import { connect } from "react-redux";
import Drawer from "react-motion-drawer";
import { isEmptyOrNull, isExisted } from "../../../functions/common";
import { dateFormat } from "../../../functions/converterTime";

class DrawerCustom extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      open: false,
      drawerStyle: `
          {
          "background": "var(--primary-dark)",
          "boxShadow" : "rgba(0, 0, 0, 0.188235) 0px 10px 20px, rgba(0, 0, 0, 0.227451) 0px 6px 6px"
          }`,
      relativeWidth: false,
      width: window.innerWidth > 500 ? 350 : 250,
      noTouchOpen: false,
      noTouchClose: false,
      collapseID: "",
    };
  }

  setDrawerStyle = (e) => {
    e.preventDefault();
    this.setState({
      drawerStyle: this.drawerStyleRef.value,
    });
  };

  toggleCollapse = (collapseID) => () =>
    this.setState((prevState) => ({
      collapseID: prevState.collapseID !== collapseID ? collapseID : "",
    }));
  render() {
    const {
      drawerStyle: stringDrawerStyle,
      open,
      noTouchOpen,
      noTouchClose,
    } = this.state;

    let drawerStyle = {};
    try {
      drawerStyle = JSON.parse(stringDrawerStyle);
    } catch (err) {
      console.error("Error parsing JSON: ", err);
    }

    const drawerProps = {
      overlayColor: "rgba(255,255,255,0.6)",
      drawerStyle,
    };

    const user = this.props.user;
    if (user) {
      if (isEmptyOrNull(user.avatar)) {
        user.avatar = require("../../../static/images/avatar-default.png");
      }
      if (isExisted(user.createdDate)) {
        user.createdAt = dateFormat(user.createdAt);
      }
    }
    return (
      <Drawer
        {...drawerProps}
        fadeOut
        width={this.state.width}
        open={open}
        onChange={(open) => this.setState({ open: open })}
        noTouchOpen={noTouchOpen}
        noTouchClose={noTouchClose}
      >
        <div className="text-center mt-4 mb-4">
          <img
            src={user.avatar}
            className="img-avatar mt-4 mb-2"
            alt="Avatar"
          />
          <p>{user.name}</p>
          <p className="reputation mb-4">
            Điểm Danh Tiếng: {user.reputation ? user.reputation : 0}
          </p>
          <div className="text-left pl-4 mt-4 pt-4">
            <p>Số điện thoại: {user.phone}</p>
            <p>Email: {user.email}</p>
            <p>Ngày tạo: {user.createdAt}</p>
          </div>
        </div>
      </Drawer>
    );
  }
}
const mapStateToProps = (state) => ({ user: state.user.userInfo.user });
const mapDispatchToProps = {};
export default connect(mapStateToProps, mapDispatchToProps)(DrawerCustom);
