import React from "react";
import Drawer from "react-motion-drawer";
import {Link} from 'react-router-dom';
import {
  Navbar,
  NavbarBrand,
  NavbarNav,
  NavItem,
  NavLink,
  Fa,
  NavbarToggler,
  Collapse,
} from "mdbreact";
// import UserAuth from '../../../api/authen'
import { isEmptyOrNull, isExisted } from "../../../functions/common";
import { dateFormat } from "../../../functions/converterTime";
import { connect } from "react-redux";
import authAction from "../../../actions/auth";
import trafficStatus from "../../../actions/traffic-status";
import { utils } from "../../../modules/utils";

class UserSideBar extends React.Component {
  constructor() {
    super();
    // this.userAuth = new UserAuth();
    this.state = {
      open: false,
      drawerStyle: `
                {
                "background": "var(--primary-dark)",
                "boxShadow" : "rgba(0, 0, 0, 0.188235) 0px 10px 20px, rgba(0, 0, 0, 0.227451) 0px 6px 6px"
                }`,
      relativeWidth: false,
      width: window.innerWidth > 500 ? 350 : 250,
      noTouchOpen: false,
      noTouchClose: false,
      collapseID: "",
    };
  }

  logout = () => {
    // this.userAuth.logout();
  };

  setWidth = (e) => {
    this.setState({
      width: Number(e.target.value) || e.target.value,
    });
  };

  setTouch = (e) => {
    this.setState({
      [e.target.name]: !e.target.checked,
    });
  };

  setDrawerStyle = (e) => {
    e.preventDefault();
    this.setState({
      drawerStyle: this.drawerStyleRef.value,
    });
  };

  toggleCollapse = (collapseID) => () =>
    this.setState((prevState) => ({
      collapseID: prevState.collapseID !== collapseID ? collapseID : "",
    }));

  render() {
    const {
      drawerStyle: stringDrawerStyle,
      open,
      noTouchOpen,
      noTouchClose,
    } = this.state;

    let drawerStyle = {};
    try {
      drawerStyle = JSON.parse(stringDrawerStyle);
    } catch (err) {
      console.error("Error parsing JSON: ", err);
    }

    const drawerProps = {
      overlayColor: "rgba(255,255,255,0.6)",
      drawerStyle,
    };

    // const user = this.props.user;
    const { user, isLogin } = this.props;
    if (user) {
      if (isEmptyOrNull(user.avatar)) {
        user.avatar = require("../../../static/images/avatar-default.png");
      }
      if (isExisted(user.createdDate)) {
        user.createdAt = dateFormat(user.createdAt);
      }
    }

    return (
      <div className="user-sidebar">
        {isLogin && (
          <Drawer
            {...drawerProps}
            fadeOut
            width={this.state.width}
            open={open}
            onChange={(open) => this.setState({ open: open })}
            noTouchOpen={noTouchOpen}
            noTouchClose={noTouchClose}
          >
            <div className="text-center mt-4 mb-4">
              <img
                src={user.avatar}
                className="img-avatar mt-4 mb-2"
                alt="Avatar"
              />
              <p>{user.name}</p>
              <p className="reputation mb-4">
                Điểm Danh Tiếng: {user.reputation ? user.reputation : 0}
              </p>
              <div className="text-left pl-4 mt-4 pt-4">
                <p>Số điện thoại: {user.phone}</p>
                <p>Email: {user.email}</p>
                <p>
                  Ngày tạo:{" "}
                  {utils.convertTimeToString(
                    user.createdAt,
                    "DD-MM-YYYY hh:mm"
                  )}
                </p>
              </div>
            </div>
          </Drawer>
        )}

        <Navbar color="cyan darken-2" dark expand="md" fixed="top">
          <NavbarBrand>
            {isLogin && (
              <span
                className="navbar-toggler-icon"
                onClick={() => this.setState({ open: !open })}
              ></span>
            )}
            <NavLink className="waves-effect waves-light" to="/">
              <strong className="white-text mr-5">UTraffic</strong>
            </NavLink>
          </NavbarBrand>

          {isLogin && (
            <NavbarToggler onClick={this.toggleCollapse("navbarCollapse3")} />
          )}

          <Collapse id="navbarCollapse3" isOpen={this.state.collapseID} navbar>
            {" "}
            <NavbarNav right>
              <NavItem>
                <NavLink
                  className="waves-effect waves-light"
                  to="/user"
                  // onClick={() => {
                  //   // this.props.changeTrafficStatusField({
                  //   //   "trafficStatus.isCanceled": true,
                  //   // });
                  //   this.props.cancelGetTrafficStatus();
                  // }}
                >
                  <span>
                    <Fa className="icon-menu" icon="exclamation-triangle" />{" "}
                    Cảnh báo giao thông
                  </span>
                </NavLink>
              </NavItem>
              {isLogin ? (
                <>
                  <NavItem>
                    <NavLink className="waves-effect waves-light" to="/user">
                      Hi, {user.name}
                    </NavLink>
                  </NavItem>
                  <NavItem>
                    <NavLink
                      className="waves-effect waves-light"
                      to="/"
                      onClick={this.props.logOut}
                    >
                      <Fa className="icon-menu" icon="power-off" />
                      Đăng xuất
                    </NavLink>
                  </NavItem>
                </>
              ) : (
                <>
                  <NavItem>
                    <Link className="waves-effect waves-light" to="/login">
                      <Fa className="icon-menu" icon="user" />
                      <span> Đăng nhập</span>
                    </Link>
                  </NavItem>
                  <NavItem>
                    <NavLink className="waves-effect waves-light" to="/signup">
                      <Fa className="icon-menu" icon="user-plus" />
                      <span> Đăng ký</span>
                    </NavLink>
                  </NavItem>
                </>
              )}
            </NavbarNav>
          </Collapse>
        </Navbar>
      </div>
    );
  }
}
const mapStateToProps = (state) => ({
  user: state.user.userInfo.user,
  isLogin: state.app.isLogin,
});

const mapDispatchToProps = {
  logOut: authAction.logout,
  // changeTrafficStatusField: trafficStatus.changeField,
  cancelGetTrafficStatus: trafficStatus.cancelGetTrafficStatus,
};

export default connect(mapStateToProps, mapDispatchToProps)(UserSideBar);
