import React from 'react'
import ProTypes from 'prop-types'
import { Sidebar, Segment, Grid, Header, Image, Menu, Icon, Checkbox, Button} from 'semantic-ui-react'

const HorizontalSideBar = ({ animation, direction, visible }) => (
    <Sidebar as={Segment} animation={animation} direction={direction} visible={visible}>
        <Grid textAlign="center">
            <Grid.Row columns={1}>
                <Grid.Column>
                    <Header as="h3">New Content</Header>
                </Grid.Column>
            </Grid.Row>
            <Grid columns={3} divided>
                <Grid.Column>
                    <Image src='https://react.semantic-ui.com/images/wireframe/media-paragraph.png' />
                </Grid.Column>
                <Grid.Column>
                    <Image src='https://react.semantic-ui.com/images/wireframe/media-paragraph.png' />
                </Grid.Column>
                <Grid.Column>
                    <Image src='https://react.semantic-ui.com/images/wireframe/media-paragraph.png' />
                </Grid.Column>
            </Grid>
        </Grid>
    </Sidebar>
)

HorizontalSideBar.proTypes = {
    animation: ProTypes.string,
    direction: ProTypes.string,
    visible: ProTypes.bool
}

const VerticalSideBar = ({ animation, direction, visible }) => (
    <Sidebar as={Menu} animation={animation} direction={direction} icon="labeled" inverted vertical visible={visible} width="thin">
        <Menu.Item as="a">
            <Icon name="home"/>Home
        </Menu.Item>
        <Menu.Item as="a">
            <Icon name="gamepad"/>Games
        </Menu.Item>
        <Menu.Item as="a">
            <Icon name="camera"/>Channels
        </Menu.Item>
    </Sidebar>
)

VerticalSideBar.proTypes = {
    animation: ProTypes.string,
    direction: ProTypes.string,
    visible: ProTypes.bool
}

export default class SideBar extends React.Component {
    constructor() {
        super();
        this.state = {
            animation: "overlay",
            direction: "left",
            dimmed: true,
            visible: false
        }
    }

    handleAnimationChange = animation => () => this.setState(preState => ({
        animation, visible: !preState.visible
    }))

    handleDimmedChange = (e, { checked }) => this.setState({ dimmed: checked })

    handleDirectionChange = direction => () => this.setState({ direction, visible: false })

    render() {
        const { animation, dimmed, direction, visible } = this.state
        // const vertical = direction === "bottom" || direction === "top"
        const vertical = true

        return(
            <div>
                <Checkbox checked={dimmed} label="Dim Page" onChange={this.handleDimmedChange} toggle />

                <Header as="h5">Direction</Header>
                <Button.Group>
                    <Button active={direction === "left"} onClick={this.handleDirectionChange("left")}>
                        Left
                    </Button>
                    <Button active={direction === "right"} onClick={this.handleDirectionChange("right")}>
                        Right
                    </Button>
                    <Button active={direction === "top"} onClick={this.handleDirectionChange("top")}>
                        Top
                    </Button>
                    <Button active={direction === "bottom"} onClick={this.handleDirectionChange("bottom")}>
                        Bottom
                    </Button>
                </Button.Group>

                <Header as="h5">All Direciton Animation</Header>
                <Button onClick={this.handleAnimationChange("overlay")}>Overlay</Button>
                <Button onClick={this.handleAnimationChange("push")}>Push</Button>
                <Button onClick={this.handleAnimationChange("scale down")}>Scale Down</Button>

                <Header as="h5">Vertical-Only Animation</Header>
                <Button disabled={vertical} onClick={this.handleAnimationChange("uncover")}>Uncover</Button>
                <Button disabled={vertical} onClick={this.handleAnimationChange("slide along")}>Slide Along</Button>
                <Button disabled={vertical} onClick={this.handleAnimationChange("slide out")}>Slide Out</Button>

                <Sidebar.Pushable as={Segment}>
                    {vertical ? (
                        <HorizontalSideBar animation={animation} direction={direction} visible={visible} />
                    ) : null}
                    {vertical ? null : (
                        <VerticalSideBar animation={animation} direction={direction} visible={visible} />
                    )}
                    <Sidebar.Pusher dimmed={dimmed && visible}>
                        <Segment basic>
                            <Header as="h3">Application Content</Header>
                            <Image src="https://react.semantic-ui.com/images/wireframe/paragraph.png" />
                        </Segment>
                    </Sidebar.Pusher>
                </Sidebar.Pushable>
            </div>
        )
    }
}