import L from 'leaflet';
import {
  updateRedirectTraffic,
  updateGeoTraffic,
} from '../functions/converterGeographic';
import {
  onEachDistance,
  onEachReportDistance,
  onEachFeature,
  weightPolyline,
  gradient,
} from '../components/map/fragments/DrawTools';
import { store } from '../App';
import redirect from '../actions/redirect';

/**
 *
 * @param {any} map
 * @param {'direct'|'trafficStatus' | 'report'} type
 */
export function clearMap(map, type) {
  console.log('Clear map');
  if (map) {
    try {
      map.eachLayer((layer) => {
        try {
          if (!type) {
            if (layer.tag && layer.tag === 'GeoJSON') {
              map.removeLayer(layer);
            }
          } else if (layer.type === type) {
            map.removeLayer(layer);
          }
        } catch (err) {
          console.log('Remove layer error', err);
        }
      });
    } catch (err) {
      console.log('Clear map error', err);
    }
  }
}
/**
 * 
 * @param {mapRef} map 
 * @param {string} type - layer's type 
 * @param {string} index  - layer's index
 * @param {boolean} isMulti - check whether is multi report or not
 */

export function clearDirectReportMap(map, type, index, isMulti) {
  if (map) {
    try {
      map.eachLayer((layer) => {
        try {
          if (isMulti && layer.type === type && layer.index === index) {
            map.removeLayer(layer);
          }else if(!isMulti && layer.type === type){
            map.removeLayer(layer);
          }
        } catch (err) {
          console.error('Remove layer error', err);
        }
      });
    } catch (err) {
      console.error('Clear map error', err);
    }
  }
}

/**
 *
 * @param {*} map
 * @param {array} redirectDataList
 * @param {*} query the params of redirect api, type is one of time / distance
 */
export function onRedirect(
  map,
  // eslint-disable-next-line no-undef
  redirectDataList,
  query,
  priority = 0
) {
  if (map) {
    try {
      map.eachLayer((layer) => {
        if (layer.type && layer.type === 'direct') {
          map.removeLayer(layer);
        }
        if (layer.type && layer.type === 'trafficStatus') {
          map.removeLayer(layer);
        }
      });
      // clearMap(map);
      // L.popup().closePopup();
      const drawedSegments = {};
      if (priority) {
        const priorityData = redirectDataList.slice(priority, priority + 1);

        redirectDataList.splice(priority, 1);
        redirectDataList.splice(0, 0, priorityData[0]);
        console.log('REDIRECT DATA ON DRAW', redirectDataList);
        store.dispatch(redirect.setState({ priority: 0 }));
      }
      redirectDataList.forEach((route, index) => {
        const { distance, time, coords, _id } = route;
        if (!coords) return;
        // const { slat, slng, elat, elng } = query;
        // console.log("QUERY", query);
        // const firstCoords = coords[0];
        // const lastCoords = coords[coords.length - 1];
        // const firstCoordsItem = {
        //   lat: slat,
        //   lng: slng,
        //   elat: firstCoords.lat,
        //   elng: firstCoords.lng,
        //   status: { color: "#1890ff" },
        //   segment_id: "start",
        // };
        // const lastCoordsItem = {
        //   lat: lastCoords.lat,
        //   lng: lastCoords.lng,
        //   elat,
        //   elng,
        //   status: { color: "#1890ff" },
        //   segment_id: "end",
        // };
        // coords.push(lastCoordsItem);
        // coords.splice(0, 0, firstCoordsItem);

        for (let i = 0; i < coords.length; i++) {
          const coordsItem = coords[i];
          const { lat, lng, elat, elng, status, segment_id } = coordsItem;
          const coordinates = [
            [lng, lat],
            [elng, elat],
          ];
          let geoJsonItem = {
            coordinates,
          };
          if (status) {
            const { color, velocity, source } = status;
            geoJsonItem = {
              index,
              color,
              velocity,
              source,
              distance,
              time,
              lat,
              lng,
              // isMiddle: i === middle ? true : false,
              segment_id,
              _id,
              ...geoJsonItem,
            };
          }

          if (!drawedSegments[segment_id]) {
            drawedSegments[segment_id] = 1;
            L.geoJSON(updateRedirectTraffic([geoJsonItem]).features, {
              style: getDirectSegmentStyle(
                query.zoom,
                query.darkmode
                // priority
              ),
              onEachFeature: onEachDistance(map),
            }).addTo(map);
          }
        }
        const { slat, slng, elat, elng } = query;
        console.log('QUERY', query);
        const firstCoords = coords[0];
        const lastCoords = coords[coords.length - 1];
        const startCoords = [
          [slng, slat],
          [firstCoords.lng, firstCoords.lat],
        ];
        const endCoords = [
          [elng, elat],
          [lastCoords.elng, lastCoords.elat],
        ];
        drawPolyline(startCoords, map, {
          style: { color: '#1890ff', dashArray: '5 5' },
          onEachFeature: (feature, layer) => {
            layer.type = 'direct';
            layer.tag = 'GeoJSON';
          },
        });
        drawPolyline(endCoords, map, {
          style: { color: index ? '#bbbdbf' : '#1890ff', dashArray: '5 5' },
          onEachFeature: (feature, layer) => {
            layer.type = 'direct';
            layer.tag = 'GeoJSON';
          },
        });
        // const lastCoordsItem = {
        //   lat: lastCoords.lat,
        //   lng: lastCoords.lng,
        //   elat,
        //   elng,
        //   status: { color: "#1890ff" },
        //   segment_id: "end",
        // };
      });
    } catch (error) {
      console.log('draw direct error', error);
    }
  }
}

/**
 *
 * @param {mapRef} map
 * @param {array} redirectDataList
 * @param {Object} query the params of redirect api, type is one of time / distance
 * @param {number} priority
 * @param {boolean} drawable
 */
export function onRedirectReport(
  map,
  // eslint-disable-next-line no-undef
  redirectDataList,
  query,
  priority = 0,
  drawable
) {
  if (map) {
    try {
      if (!drawable) {
        map.eachLayer((layer) => {
          if (layer.type && layer.type === 'report-direct') {
            map.removeLayer(layer);
          }
          if (layer.type && layer.type === 'trafficStatus') {
            map.removeLayer(layer);
          }
        });
      }
      const drawedSegments = {};
      redirectDataList.splice(1, 1);
      redirectDataList.forEach((route, index) => {
        const { distance, time, coords } = route;
        if (!coords) return;
        for (let i = 0; i < coords.length; i++) {
          const coordsItem = coords[i];
          const { lat, lng, elat, elng, status, segment_id } = coordsItem;
          const coordinates = [
            [lng, lat],
            [elng, elat],
          ];
          let geoJsonItem = {
            coordinates,
          };
          if (status) {
            const { color, velocity, source } = status;
            geoJsonItem = {
              index,
              color,
              velocity,
              source,
              distance,
              time,
              lat,
              lng,
              segment_id,
              ...geoJsonItem,
            };
          }

          if (!drawedSegments[segment_id]) {
            drawedSegments[segment_id] = 1;
            L.geoJSON(updateRedirectTraffic([geoJsonItem]).features, {
              style: getDirectSegmentStyle(query.zoom, query.darkmode, true),
              onEachFeature: onEachReportDistance(map),
            }).addTo(map);
          }
        }
        if (!drawable) {
          const { slat, slng, elat, elng } = query;
          const firstCoords = coords[0];
          const lastCoords = coords[coords.length - 1];
          const startCoords = [
            [slng, slat],
            [firstCoords.lng, firstCoords.lat],
          ];
          const endCoords = [
            [elng, elat],
            [lastCoords.elng, lastCoords.elat],
          ];
          drawPolyline(startCoords, map, {
            style: { color: '#1890ff', dashArray: '5 5' },
            onEachFeature: (feature, layer) => {
              layer.type = 'report-direct';
              layer.tag = 'GeoJSON';
            },
          });
          drawPolyline(endCoords, map, {
            style: { color: index ? '#bbbdbf' : '#1890ff', dashArray: '5 5' },
            onEachFeature: (feature, layer) => {
              layer.type = 'report-direct';
              layer.tag = 'GeoJSON';
            },
          });
        }
      });
    } catch (error) {
      console.log('draw direct error', error);
    }
  }
}
/**
 *
 * @param {*} zoom : zoomLevel;
 * @param {*} darkmode : mode
 * @param {*} isReport : whereas normal direct or direct for report
 * @returns
 */

const getDirectSegmentStyle =
  (zoom, darkmode, isReport = false) =>
  (feature) => {
    // const v = feature.properties.v;
    const {
      color: baseColor,
      velocity: v,
      index,
      segment_id,
    } = feature.properties;
    let color = baseColor;
    // if(segment_id === 'start')
    if (index) {
      color = '#bbbdbf';
    } else {
      if (v >= 40) {
        color = '#1890ff';
      } else {
        color = gradient(v / 60);
        // color = baseColor;
      }
    }
    const style = {
      opacity: 1,
      // weight: weightPolyline(zoom, darkmode),
      weight: isReport ? 6 : 7,
      color,
    };
    if (segment_id === 'start' || segment_id === 'end') {
      style.dashArray = ' 5 10';
    }
    return style;
  };

const style = (zoom, darkmode) => (feature) => {
  return {
    opacity: 1,
    weight: weightPolyline(zoom, darkmode),
    // weight: 7,
    color: gradient(feature.properties.v / 60),
  };
};

/**
 *
 * @param {*} map
 * @param {*} query the params of get traffic status
 */
export function onGetTrafficStatus(
  map,
  // eslint-disable-next-line no-undef
  data,
  zoom,
  darkmode
) {
  const startTime = Date.now();
  if (map && data.length) {
    // map.eachLayer((layer) => {
    //   if (layer.type && layer.type === "trafficStatus") {
    //     map.removeLayer(layer);
    //   }
    // });
    clearMap(map);
    const dataLength = data.length;
    const threadCount = 4;
    const subLength = Math.ceil(dataLength / threadCount);
    const subDataList = [];
    for (let i = 0; i < threadCount; i++) {
      const subData = data.slice(
        i * subLength,
        i === threadCount - 1 ? dataLength : (i + 1) * subLength
      );
      subDataList.push(subData);
    }
    subDataList.forEach((item, index) => {
      leafletGeoJson(item, zoom, darkmode, map, index);
    });
    // leafletGeoJson(data, zoom, darkmode, map,);
  }
  console.log('handle get traffic status time', Date.now() - startTime);
}

const leafletGeoJson = async (data, zoom, darkmode, map, i) => {
  if (!map) {
    return;
  }
  try {
    L.geoJSON(updateGeoTraffic(data, i).features, {
      style: style(zoom, darkmode),
      onEachFeature: onEachFeature,
    }).addTo(map);
  } catch (err) {
    console.log('Add to map error', err);
  }
};

/**
 *
 *
 * @param {array[lng,lat]} coordinates
 * @param {*} map
 * @param {L.GeoJSONOptions<any>} param2
 */
export const drawPolyline = (
  coordinates,
  map,
  { style = { weight: 3, color: 'red' }, ...options },
  geometryType = 'LineString'
) => {
  if (map) {
    // console.log("DRAW POLYLINE", coordinates);
    const feature = {
      type: 'Feature',
      properties: {},
      geometry: {
        type: geometryType,
        coordinates,
      },
    };
    L.geoJSON([feature], {
      style,
      ...options,
    }).addTo(map);
  }
  // L.geoJSON({},{style,onEachFeature: (feature, layer)=>{
  //   layer.on({
  //     mouseout:(event)=>{
  //       event.
  //     }
  //   })
  // }})
};

export const mapFunctions = {
  clearMap,
  clearDirectReportMap,
};
