import errorDefined from "./error-defined";
import config from "./config";

/**
 *@param {string} message
 * @param {{domain:string,message:string,reason:string}[]} errors
 */
function getMessageErrors(message, errors) {
  let notice = "";
  if (!errors) {
    return { message: "Lỗi chưa xác định", notice: "" };
  }
  errors.forEach((item) => {
    const { domain } = item;
    notice += `${errorDefined[domain]} ${errorDefined[message]} `;
  });
  message = errorDefined[message];
  return { message, notice };
}

/**
 *
 * @param {[number,number]} firstPoint
 * @param {[number,number]} secondPoint
 */
const getDistanceBetweenToPoint = (firstPoint, secondPoint) => {
  const deltaX = firstPoint[0] - secondPoint[0];
  const deltaY = firstPoint[1] - secondPoint[1];
  const distance = Math.sqrt(Math.pow(deltaX, 2) + Math.pow(deltaY, 2));
  return distance;
};

const convertTimeToString = (time, format) => {
  const date = new Date(time);
  const year = date.getFullYear();
  const mon = ("0" + (date.getMonth() + 1)).slice(-2);
  const day = ("0" + date.getDate()).slice(-2);
  const hour = ("0" + date.getHours()).slice(-2);
  const min = ("0" + date.getMinutes()).slice(-2);
  const second = ("0" + date.getSeconds()).slice(-2);

  return format
    .replace("YYYY", year)
    .replace("yyyy", year)
    .replace("dd", day)
    .replace("DD", day)
    .replace("MM", mon)
    .replace("hh", hour)
    .replace("mm", min)
    .replace("ss", second);
};

/**
 *
 * @param {Coordinate} coord1
 * @param {Coordinate} coord2
 */
function getDistanceBetweenTwoCoords(coord1, coord2) {
  const lat = Math.pow(parseFloat(coord1.lat) - parseFloat(coord2.lat), 2);
  const lng = Math.pow(parseFloat(coord1.lng) - parseFloat(coord2.lng), 2);
  const dis = Math.sqrt(lat + lng);
  return Math.round((dis * config.EARTH_CIRCUMFERENCE) / 360);
}

export const utils = {
  getMessageErrors,
  getDistanceBetweenToPoint,
  convertTimeToString,
  getDistanceBetweenTwoCoords,
};
